@@ -8,8 +8,28 @@ import (
 
 	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/cancel"
+	"github.com/lxc/lxd/shared/ioprogress"
 )
 
+// The Operation type represents a currently running operation.
+type Operation interface {
+	AddHandler(function func(api.Operation)) (target *EventTarget, err error)
+	Cancel() (err error)
+	Get() (op api.Operation)
+	GetWebsocket(secret string) (conn *websocket.Conn, err error)
+	RemoveHandler(target *EventTarget) (err error)
+	Refresh() (err error)
+	Wait() (err error)
+}
+
+// The RemoteOperation type represents an Operation that may be using multiple servers.
+type RemoteOperation interface {
+	AddHandler(function func(api.Operation)) (target *EventTarget, err error)
+	CancelTarget() (err error)
+	GetTarget() (op *api.Operation, err error)
+	Wait() (err error)
+}
+
 // The Server type represents a generic read-only server.
 type Server interface {
 	GetConnectionInfo() (info *ConnectionInfo, err error)
@@ -47,6 +67,8 @@ type ContainerServer interface {
 	UpdateServer(server api.ServerPut, ETag string) (err error)
 	HasExtension(extension string) (exists bool)
 	RequireAuthenticated(authenticated bool)
+	IsClustered() (clustered bool)
+	UseTarget(name string) (client ContainerServer)
 
 	// Certificate functions
 	GetCertificateFingerprints() (fingerprints []string, err error)
@@ -60,16 +82,16 @@ type ContainerServer interface {
 	GetContainerNames() (names []string, err error)
 	GetContainers() (containers []api.Container, err error)
 	GetContainer(name string) (container *api.Container, ETag string, err error)
-	CreateContainer(container api.ContainersPost) (op *Operation, err error)
-	CreateContainerFromImage(source ImageServer, image api.Image, imgcontainer api.ContainersPost) (op *RemoteOperation, err error)
-	CopyContainer(source ContainerServer, container api.Container, args *ContainerCopyArgs) (op *RemoteOperation, err error)
-	UpdateContainer(name string, container api.ContainerPut, ETag string) (op *Operation, err error)
-	RenameContainer(name string, container api.ContainerPost) (op *Operation, err error)
-	MigrateContainer(name string, container api.ContainerPost) (op *Operation, err error)
-	DeleteContainer(name string) (op *Operation, err error)
+	CreateContainer(container api.ContainersPost) (op Operation, err error)
+	CreateContainerFromImage(source ImageServer, image api.Image, imgcontainer api.ContainersPost) (op RemoteOperation, err error)
+	CopyContainer(source ContainerServer, container api.Container, args *ContainerCopyArgs) (op RemoteOperation, err error)
+	UpdateContainer(name string, container api.ContainerPut, ETag string) (op Operation, err error)
+	RenameContainer(name string, container api.ContainerPost) (op Operation, err error)
+	MigrateContainer(name string, container api.ContainerPost) (op Operation, err error)
+	DeleteContainer(name string) (op Operation, err error)
 
-	ExecContainer(containerName string, exec api.ContainerExecPost, args *ContainerExecArgs) (op *Operation, err error)
-	ConsoleContainer(containerName string, console api.ContainerConsolePost, args *ContainerConsoleArgs) (op *Operation, err error)
+	ExecContainer(containerName string, exec api.ContainerExecPost, args *ContainerExecArgs) (op Operation, err error)
+	ConsoleContainer(containerName string, console api.ContainerConsolePost, args *ContainerConsoleArgs) (op Operation, err error)
 	GetContainerConsoleLog(containerName string, args *ContainerConsoleLogArgs) (content io.ReadCloser, err error)
 	DeleteContainerConsoleLog(containerName string, args *ContainerConsoleLogArgs) (err error)
 
@@ -80,14 +102,14 @@ type ContainerServer interface {
 	GetContainerSnapshotNames(containerName string) (names []string, err error)
 	GetContainerSnapshots(containerName string) (snapshots []api.ContainerSnapshot, err error)
 	GetContainerSnapshot(containerName string, name string) (snapshot *api.ContainerSnapshot, ETag string, err error)
-	CreateContainerSnapshot(containerName string, snapshot api.ContainerSnapshotsPost) (op *Operation, err error)
-	CopyContainerSnapshot(source ContainerServer, snapshot api.ContainerSnapshot, args *ContainerSnapshotCopyArgs) (op *RemoteOperation, err error)
-	RenameContainerSnapshot(containerName string, name string, container api.ContainerSnapshotPost) (op *Operation, err error)
-	MigrateContainerSnapshot(containerName string, name string, container api.ContainerSnapshotPost) (op *Operation, err error)
-	DeleteContainerSnapshot(containerName string, name string) (op *Operation, err error)
+	CreateContainerSnapshot(containerName string, snapshot api.ContainerSnapshotsPost) (op Operation, err error)
+	CopyContainerSnapshot(source ContainerServer, snapshot api.ContainerSnapshot, args *ContainerSnapshotCopyArgs) (op RemoteOperation, err error)
+	RenameContainerSnapshot(containerName string, name string, container api.ContainerSnapshotPost) (op Operation, err error)
+	MigrateContainerSnapshot(containerName string, name string, container api.ContainerSnapshotPost) (op Operation, err error)
+	DeleteContainerSnapshot(containerName string, name string) (op Operation, err error)
 
 	GetContainerState(name string) (state *api.ContainerState, ETag string, err error)
-	UpdateContainerState(name string, state api.ContainerStatePut, ETag string) (op *Operation, err error)
+	UpdateContainerState(name string, state api.ContainerStatePut, ETag string) (op Operation, err error)
 
 	GetContainerLogfiles(name string) (logfiles []string, err error)
 	GetContainerLogfile(name string, filename string) (content io.ReadCloser, err error)
@@ -106,12 +128,12 @@ type ContainerServer interface {
 	GetEvents() (listener *EventListener, err error)
 
 	// Image functions
-	CreateImage(image api.ImagesPost, args *ImageCreateArgs) (op *Operation, err error)
-	CopyImage(source ImageServer, image api.Image, args *ImageCopyArgs) (op *RemoteOperation, err error)
+	CreateImage(image api.ImagesPost, args *ImageCreateArgs) (op Operation, err error)
+	CopyImage(source ImageServer, image api.Image, args *ImageCopyArgs) (op RemoteOperation, err error)
 	UpdateImage(fingerprint string, image api.ImagePut, ETag string) (err error)
-	DeleteImage(fingerprint string) (op *Operation, err error)
-	RefreshImage(fingerprint string) (op *Operation, err error)
-	CreateImageSecret(fingerprint string) (op *Operation, err error)
+	DeleteImage(fingerprint string) (op Operation, err error)
+	RefreshImage(fingerprint string) (op Operation, err error)
+	CreateImageSecret(fingerprint string) (op Operation, err error)
 	CreateImageAlias(alias api.ImageAliasesPost) (err error)
 	UpdateImageAlias(name string, alias api.ImageAliasesEntryPut, ETag string) (err error)
 	RenameImageAlias(name string, alias api.ImageAliasesEntryPost) (err error)
@@ -121,6 +143,7 @@ type ContainerServer interface {
 	GetNetworkNames() (names []string, err error)
 	GetNetworks() (networks []api.Network, err error)
 	GetNetwork(name string) (network *api.Network, ETag string, err error)
+	GetNetworkLeases(name string) (leases []api.NetworkLease, err error)
 	CreateNetwork(network api.NetworksPost) (err error)
 	UpdateNetwork(name string, network api.NetworkPut, ETag string) (err error)
 	RenameNetwork(name string, network api.NetworkPost) (err error)
@@ -159,10 +182,23 @@ type ContainerServer interface {
 	UpdateStoragePoolVolume(pool string, volType string, name string, volume api.StorageVolumePut, ETag string) (err error)
 	DeleteStoragePoolVolume(pool string, volType string, name string) (err error)
 	RenameStoragePoolVolume(pool string, volType string, name string, volume api.StorageVolumePost) (err error)
+	CopyStoragePoolVolume(pool string, source ContainerServer, sourcePool string, volume api.StorageVolume, args *StoragePoolVolumeCopyArgs) (op RemoteOperation, err error)
+	MoveStoragePoolVolume(pool string, source ContainerServer, sourcePool string, volume api.StorageVolume, args *StoragePoolVolumeMoveArgs) (op RemoteOperation, err error)
+	MigrateStoragePoolVolume(pool string, volume api.StorageVolumePost) (op Operation, err error)
+
+	// Cluster functions ("cluster" API extensions)
+	GetCluster() (cluster *api.Cluster, ETag string, err error)
+	UpdateCluster(cluster api.ClusterPut, ETag string) (op Operation, err error)
+	DeleteClusterMember(name string, force bool) (err error)
+	GetClusterMemberNames() (names []string, err error)
+	GetClusterMembers() (members []api.ClusterMember, err error)
+	GetClusterMember(name string) (member *api.ClusterMember, ETag string, err error)
+	RenameClusterMember(name string, member api.ClusterMemberPost) (err error)
 
 	// Internal functions (for internal use)
 	RawQuery(method string, path string, data interface{}, queryETag string) (resp *api.Response, ETag string, err error)
 	RawWebsocket(path string) (conn *websocket.Conn, err error)
+	RawOperation(method string, path string, data interface{}, queryETag string) (op Operation, ETag string, err error)
 }
 
 // The ConnectionInfo struct represents general information for a connection
@@ -172,21 +208,6 @@ type ConnectionInfo struct {
 	Protocol    string
 }
 
-// The ProgressData struct represents new progress information on an operation
-type ProgressData struct {
-	// Preferred string repreentation of progress (always set)
-	Text string
-
-	// Progress in percent
-	Percentage int
-
-	// Number of bytes transferred (for files)
-	TransferredBytes int64
-
-	// Total number of bytes (for files)
-	TotalBytes int64
-}
-
 // The ImageCreateArgs struct is used for direct image upload
 type ImageCreateArgs struct {
 	// Reader for the meta file
@@ -202,7 +223,7 @@ type ImageCreateArgs struct {
 	RootfsName string
 
 	// Progress handler (called with upload progress)
-	ProgressHandler func(progress ProgressData)
+	ProgressHandler func(progress ioprogress.ProgressData)
 }
 
 // The ImageFileRequest struct is used for an image download request
@@ -214,7 +235,7 @@ type ImageFileRequest struct {
 	RootfsFile io.WriteSeeker
 
 	// Progress handler (called whenever some progress is made)
-	ProgressHandler func(progress ProgressData)
+	ProgressHandler func(progress ioprogress.ProgressData)
 
 	// A canceler that can be used to interrupt some part of the image download request
 	Canceler *cancel.Canceler
@@ -254,6 +275,22 @@ type ImageCopyArgs struct {
 	Public bool
 }
 
+// The StoragePoolVolumeCopyArgs struct is used to pass additional options
+// during storage volume copy
+type StoragePoolVolumeCopyArgs struct {
+	// New name for the target
+	Name string
+
+	// The transfer mode, can be "pull" (default), "push" or "relay"
+	Mode string
+}
+
+// The StoragePoolVolumeMoveArgs struct is used to pass additional options
+// during storage volume move
+type StoragePoolVolumeMoveArgs struct {
+	StoragePoolVolumeCopyArgs
+}
+
 // The ContainerCopyArgs struct is used to pass additional options during container copy
 type ContainerCopyArgs struct {
 	// If set, the container will be renamed on copy
