@@ -35,6 +35,8 @@ type ProtocolLXD struct {
 	bakeryClient         *httpbakery.Client
 	bakeryInteractor     httpbakery.Interactor
 	requireAuthenticated bool
+
+	clusterTarget string
 }
 
 // GetConnectionInfo returns the basic connection information used to interact with the server
@@ -48,7 +50,7 @@ func (r *ProtocolLXD) GetConnectionInfo() (*ConnectionInfo, error) {
 		urls = append(urls, r.httpHost)
 	}
 
-	if len(r.server.Environment.Addresses) > 0 {
+	if r.server != nil && len(r.server.Environment.Addresses) > 0 {
 		for _, addr := range r.server.Environment.Addresses {
 			url := fmt.Sprintf("https://%s", addr)
 			if !shared.StringInSlice(url, urls) {
@@ -110,6 +112,12 @@ func (r *ProtocolLXD) RawWebsocket(path string) (*websocket.Conn, error) {
 	return r.websocket(path)
 }
 
+// RawOperation allows direct querying of a LXD API endpoint returning
+// background operations.
+func (r *ProtocolLXD) RawOperation(method string, path string, data interface{}, ETag string) (Operation, string, error) {
+	return r.queryOperation(method, path, data, ETag)
+}
+
 // Internal functions
 func (r *ProtocolLXD) parseResponse(resp *http.Response) (*api.Response, string, error) {
 	// Get the ETag
@@ -227,7 +235,7 @@ func (r *ProtocolLXD) queryStruct(method string, path string, data interface{},
 	return etag, nil
 }
 
-func (r *ProtocolLXD) queryOperation(method string, path string, data interface{}, ETag string) (*Operation, string, error) {
+func (r *ProtocolLXD) queryOperation(method string, path string, data interface{}, ETag string) (Operation, string, error) {
 	// Attempt to setup an early event listener
 	listener, err := r.GetEvents()
 	if err != nil {
@@ -255,7 +263,7 @@ func (r *ProtocolLXD) queryOperation(method string, path string, data interface{
 	}
 
 	// Setup an Operation wrapper
-	op := Operation{
+	op := operation{
 		Operation: *respOperation,
 		r:         r,
 		listener:  listener,
