@@ -10,6 +10,10 @@ import (
 
 // GetNetworkNames returns a list of network names
 func (r *ProtocolLXD) GetNetworkNames() ([]string, error) {
+	if !r.HasExtension("network") {
+		return nil, fmt.Errorf("The server is missing the required \"network\" API extension")
+	}
+
 	urls := []string{}
 
 	// Fetch the raw value
@@ -30,6 +34,10 @@ func (r *ProtocolLXD) GetNetworkNames() ([]string, error) {
 
 // GetNetworks returns a list of Network struct
 func (r *ProtocolLXD) GetNetworks() ([]api.Network, error) {
+	if !r.HasExtension("network") {
+		return nil, fmt.Errorf("The server is missing the required \"network\" API extension")
+	}
+
 	networks := []api.Network{}
 
 	// Fetch the raw value
@@ -43,10 +51,18 @@ func (r *ProtocolLXD) GetNetworks() ([]api.Network, error) {
 
 // GetNetwork returns a Network entry for the provided name
 func (r *ProtocolLXD) GetNetwork(name string) (*api.Network, string, error) {
+	if !r.HasExtension("network") {
+		return nil, "", fmt.Errorf("The server is missing the required \"network\" API extension")
+	}
+
 	network := api.Network{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct("GET", fmt.Sprintf("/networks/%s", url.QueryEscape(name)), nil, "", &network)
+	path := fmt.Sprintf("/networks/%s", url.QueryEscape(name))
+	if r.clusterTarget != "" {
+		path += fmt.Sprintf("?target=%s", r.clusterTarget)
+	}
+	etag, err := r.queryStruct("GET", path, nil, "", &network)
 	if err != nil {
 		return nil, "", err
 	}
@@ -54,6 +70,23 @@ func (r *ProtocolLXD) GetNetwork(name string) (*api.Network, string, error) {
 	return &network, etag, nil
 }
 
+// GetNetworkLeases returns a list of Network struct
+func (r *ProtocolLXD) GetNetworkLeases(name string) ([]api.NetworkLease, error) {
+	if !r.HasExtension("network_leases") {
+		return nil, fmt.Errorf("The server is missing the required \"network_leases\" API extension")
+	}
+
+	leases := []api.NetworkLease{}
+
+	// Fetch the raw value
+	_, err := r.queryStruct("GET", fmt.Sprintf("/networks/%s/leases", url.QueryEscape(name)), nil, "", &leases)
+	if err != nil {
+		return nil, err
+	}
+
+	return leases, nil
+}
+
 // CreateNetwork defines a new network using the provided Network struct
 func (r *ProtocolLXD) CreateNetwork(network api.NetworksPost) error {
 	if !r.HasExtension("network") {
@@ -61,7 +94,11 @@ func (r *ProtocolLXD) CreateNetwork(network api.NetworksPost) error {
 	}
 
 	// Send the request
-	_, _, err := r.query("POST", "/networks", network, "")
+	path := "/networks"
+	if r.clusterTarget != "" {
+		path += fmt.Sprintf("?target=%s", r.clusterTarget)
+	}
+	_, _, err := r.query("POST", path, network, "")
 	if err != nil {
 		return err
 	}
@@ -71,6 +108,10 @@ func (r *ProtocolLXD) CreateNetwork(network api.NetworksPost) error {
 
 // UpdateNetwork updates the network to match the provided Network struct
 func (r *ProtocolLXD) UpdateNetwork(name string, network api.NetworkPut, ETag string) error {
+	if !r.HasExtension("network") {
+		return fmt.Errorf("The server is missing the required \"network\" API extension")
+	}
+
 	// Send the request
 	_, _, err := r.query("PUT", fmt.Sprintf("/networks/%s", url.QueryEscape(name)), network, ETag)
 	if err != nil {
@@ -82,6 +123,10 @@ func (r *ProtocolLXD) UpdateNetwork(name string, network api.NetworkPut, ETag st
 
 // RenameNetwork renames an existing network entry
 func (r *ProtocolLXD) RenameNetwork(name string, network api.NetworkPost) error {
+	if !r.HasExtension("network") {
+		return fmt.Errorf("The server is missing the required \"network\" API extension")
+	}
+
 	// Send the request
 	_, _, err := r.query("POST", fmt.Sprintf("/networks/%s", url.QueryEscape(name)), network, "")
 	if err != nil {
@@ -93,6 +138,10 @@ func (r *ProtocolLXD) RenameNetwork(name string, network api.NetworkPost) error
 
 // DeleteNetwork deletes an existing network
 func (r *ProtocolLXD) DeleteNetwork(name string) error {
+	if !r.HasExtension("network") {
+		return fmt.Errorf("The server is missing the required \"network\" API extension")
+	}
+
 	// Send the request
 	_, _, err := r.query("DELETE", fmt.Sprintf("/networks/%s", url.QueryEscape(name)), nil, "")
 	if err != nil {
