@@ -36,6 +36,10 @@ func (r *ProtocolLXD) GetStoragePoolNames() ([]string, error) {
 
 // GetStoragePools returns a list of StoragePool entries
 func (r *ProtocolLXD) GetStoragePools() ([]api.StoragePool, error) {
+	if !r.HasExtension("storage") {
+		return nil, fmt.Errorf("The server is missing the required \"storage\" API extension")
+	}
+
 	pools := []api.StoragePool{}
 
 	// Fetch the raw value
@@ -49,10 +53,18 @@ func (r *ProtocolLXD) GetStoragePools() ([]api.StoragePool, error) {
 
 // GetStoragePool returns a StoragePool entry for the provided pool name
 func (r *ProtocolLXD) GetStoragePool(name string) (*api.StoragePool, string, error) {
+	if !r.HasExtension("storage") {
+		return nil, "", fmt.Errorf("The server is missing the required \"storage\" API extension")
+	}
+
 	pool := api.StoragePool{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct("GET", fmt.Sprintf("/storage-pools/%s", url.QueryEscape(name)), nil, "", &pool)
+	path := fmt.Sprintf("/storage-pools/%s", url.QueryEscape(name))
+	if r.clusterTarget != "" {
+		path += fmt.Sprintf("?target=%s", r.clusterTarget)
+	}
+	etag, err := r.queryStruct("GET", path, nil, "", &pool)
 	if err != nil {
 		return nil, "", err
 	}
@@ -71,7 +83,11 @@ func (r *ProtocolLXD) CreateStoragePool(pool api.StoragePoolsPost) error {
 	}
 
 	// Send the request
-	_, _, err := r.query("POST", "/storage-pools", pool, "")
+	path := "/storage-pools"
+	if r.clusterTarget != "" {
+		path += fmt.Sprintf("?target=%s", r.clusterTarget)
+	}
+	_, _, err := r.query("POST", path, pool, "")
 	if err != nil {
 		return err
 	}
@@ -81,6 +97,10 @@ func (r *ProtocolLXD) CreateStoragePool(pool api.StoragePoolsPost) error {
 
 // UpdateStoragePool updates the pool to match the provided StoragePool struct
 func (r *ProtocolLXD) UpdateStoragePool(name string, pool api.StoragePoolPut, ETag string) error {
+	if !r.HasExtension("storage") {
+		return fmt.Errorf("The server is missing the required \"storage\" API extension")
+	}
+
 	// Send the request
 	_, _, err := r.query("PUT", fmt.Sprintf("/storage-pools/%s", url.QueryEscape(name)), pool, ETag)
 	if err != nil {
@@ -92,6 +112,10 @@ func (r *ProtocolLXD) UpdateStoragePool(name string, pool api.StoragePoolPut, ET
 
 // DeleteStoragePool deletes a storage pool
 func (r *ProtocolLXD) DeleteStoragePool(name string) error {
+	if !r.HasExtension("storage") {
+		return fmt.Errorf("The server is missing the required \"storage\" API extension")
+	}
+
 	// Send the request
 	_, _, err := r.query("DELETE", fmt.Sprintf("/storage-pools/%s", url.QueryEscape(name)), nil, "")
 	if err != nil {
