@@ -3,7 +3,7 @@ package main
 import (
 	"testing"
 
-	"github.com/lxc/lxd"
+	"github.com/lxc/lxd/lxc/config"
 )
 
 type aliasTestcase struct {
@@ -40,21 +40,21 @@ func TestExpandAliases(t *testing.T) {
 	}
 
 	testcases := []aliasTestcase{
-		aliasTestcase{
+		{
 			input:    []string{"lxc", "list"},
 			expected: []string{"lxc", "list"},
 		},
-		aliasTestcase{
+		{
 			input:    []string{"lxc", "tester", "12"},
-			expected: []string{"lxc", "list", "--no-alias"},
+			expected: []string{"lxc", "list"},
 		},
-		aliasTestcase{
+		{
 			input:    []string{"lxc", "foo", "asdf"},
-			expected: []string{"lxc", "list", "--no-alias", "asdf", "-c", "n"},
+			expected: []string{"lxc", "list", "asdf", "-c", "n"},
 		},
 	}
 
-	conf := &lxd.Config{Aliases: aliases}
+	conf := &config.Config{Aliases: aliases}
 
 	for _, tc := range testcases {
 		result, expanded := expandAlias(conf, tc.input)
