@@ -28,10 +28,9 @@ var apiInternal = []Command{
 	internalContainerOnStartCmd,
 	internalContainerOnStopCmd,
 	internalContainersCmd,
-}
-
-func internalReady(d *Daemon, r *http.Request) Response {
-	return InternalError(fmt.Errorf("The server does not support setup mode"))
+	internalSQLCmd,
+	internalClusterAcceptCmd,
+	internalClusterContainerMovedCmd,
 }
 
 func internalWaitReady(d *Daemon, r *http.Request) Response {
@@ -41,7 +40,7 @@ func internalWaitReady(d *Daemon, r *http.Request) Response {
 }
 
 func internalShutdown(d *Daemon, r *http.Request) Response {
-	d.shutdownChan <- true
+	d.shutdownChan <- struct{}{}
 
 	return EmptySyncResponse
 }
@@ -91,10 +90,78 @@ func internalContainerOnStop(d *Daemon, r *http.Request) Response {
 	return EmptySyncResponse
 }
 
+type internalSQLPost struct {
+	Query string `json:"query" yaml:"query"`
+}
+
+type internalSQLResult struct {
+	Columns      []string        `json:"columns" yaml:"columns"`
+	Rows         [][]interface{} `json:"rows" yaml:"rows"`
+	RowsAffected int64           `json:"rows_affected" yaml:"rows_affected"`
+}
+
+func internalSQL(d *Daemon, r *http.Request) Response {
+	req := &internalSQLPost{}
+	// Parse the request.
+	err := json.NewDecoder(r.Body).Decode(&req)
+	if err != nil {
+		return BadRequest(err)
+	}
+	db := d.cluster.DB()
+	result := internalSQLResult{}
+	if strings.HasPrefix(strings.ToUpper(req.Query), "SELECT") {
+		rows, err := db.Query(req.Query)
+		if err != nil {
+			return SmartError(err)
+		}
+		defer rows.Close()
+		result.Columns, err = rows.Columns()
+		if err != nil {
+			return SmartError(err)
+		}
+		for rows.Next() {
+			row := make([]interface{}, len(result.Columns))
+			rowPointers := make([]interface{}, len(result.Columns))
+			for i := range row {
+				rowPointers[i] = &row[i]
+			}
+			err := rows.Scan(rowPointers...)
+			if err != nil {
+				return SmartError(err)
+			}
+			for i, column := range row {
+				// Convert bytes to string. This is safe as
+				// long as we don't have any BLOB column type.
+				data, ok := column.([]byte)
+				if ok {
+					row[i] = string(data)
+				}
+			}
+			result.Rows = append(result.Rows, row)
+		}
+		err = rows.Err()
+		if err != nil {
+			return SmartError(err)
+		}
+	} else {
+		r, err := db.Exec(req.Query)
+		if err != nil {
+			return SmartError(err)
+		}
+		result.RowsAffected, err = r.RowsAffected()
+		if err != nil {
+			return SmartError(err)
+		}
+
+	}
+	return SyncResponse(true, result)
+}
+
 var internalShutdownCmd = Command{name: "shutdown", put: internalShutdown}
-var internalReadyCmd = Command{name: "ready", put: internalReady, get: internalWaitReady}
+var internalReadyCmd = Command{name: "ready", get: internalWaitReady}
 var internalContainerOnStartCmd = Command{name: "containers/{id}/onstart", get: internalContainerOnStart}
 var internalContainerOnStopCmd = Command{name: "containers/{id}/onstop", get: internalContainerOnStop}
+var internalSQLCmd = Command{name: "sql", post: internalSQL}
 
 func slurpBackupFile(path string) (*backupFile, error) {
 	data, err := ioutil.ReadFile(path)
@@ -188,9 +255,9 @@ func internalImport(d *Daemon, r *http.Request) Response {
 
 	// Try to retrieve the storage pool the container supposedly lives on.
 	var poolErr error
-	poolID, pool, poolErr := d.db.StoragePoolGet(containerPoolName)
+	poolID, pool, poolErr := d.cluster.StoragePoolGet(containerPoolName)
 	if poolErr != nil {
-		if poolErr != db.NoSuchObjectError {
+		if poolErr != db.ErrNoSuchObject {
 			return SmartError(poolErr)
 		}
 	}
@@ -202,7 +269,7 @@ func internalImport(d *Daemon, r *http.Request) Response {
 			`recovered manually`))
 	}
 
-	if poolErr == db.NoSuchObjectError {
+	if poolErr == db.ErrNoSuchObject {
 		// Create the storage pool db entry if it doesn't exist.
 		err := storagePoolDBCreate(d.State(), containerPoolName, "",
 			backup.Pool.Driver, backup.Pool.Config)
@@ -210,7 +277,7 @@ func internalImport(d *Daemon, r *http.Request) Response {
 			return SmartError(err)
 		}
 
-		poolID, err = d.db.StoragePoolGetID(containerPoolName)
+		poolID, err = d.cluster.StoragePoolGetID(containerPoolName)
 		if err != nil {
 			return SmartError(err)
 		}
@@ -314,7 +381,7 @@ func internalImport(d *Daemon, r *http.Request) Response {
 				onDiskPoolName, req.Name,
 				storagePoolVolumeTypeNameContainer, userName)
 			if err != nil {
-				if err != db.NoSuchObjectError {
+				if err != db.ErrNoSuchObject {
 					return InternalError(err)
 				}
 			}
@@ -505,10 +572,10 @@ func internalImport(d *Daemon, r *http.Request) Response {
 	}
 
 	// Check if a storage volume entry for the container already exists.
-	_, volume, ctVolErr := d.db.StoragePoolVolumeGetType(
+	_, volume, ctVolErr := d.cluster.StoragePoolNodeVolumeGetType(
 		req.Name, storagePoolVolumeTypeContainer, poolID)
 	if ctVolErr != nil {
-		if ctVolErr != db.NoSuchObjectError {
+		if ctVolErr != db.ErrNoSuchObject {
 			return SmartError(ctVolErr)
 		}
 	}
@@ -520,7 +587,7 @@ func internalImport(d *Daemon, r *http.Request) Response {
 	}
 
 	// Check if an entry for the container already exists in the db.
-	_, containerErr := d.db.ContainerId(req.Name)
+	_, containerErr := d.cluster.ContainerID(req.Name)
 	if containerErr != nil {
 		if containerErr != sql.ErrNoRows {
 			return SmartError(containerErr)
@@ -555,7 +622,7 @@ func internalImport(d *Daemon, r *http.Request) Response {
 
 		// Remove the storage volume db entry for the container since
 		// force was specified.
-		err := d.db.StoragePoolVolumeDelete(req.Name,
+		err := d.cluster.StoragePoolVolumeDelete(req.Name,
 			storagePoolVolumeTypeContainer, poolID)
 		if err != nil {
 			return SmartError(err)
@@ -565,7 +632,7 @@ func internalImport(d *Daemon, r *http.Request) Response {
 	if containerErr == nil {
 		// Remove the storage volume db entry for the container since
 		// force was specified.
-		err := d.db.ContainerRemove(req.Name)
+		err := d.cluster.ContainerRemove(req.Name)
 		if err != nil {
 			return SmartError(err)
 		}
@@ -573,7 +640,7 @@ func internalImport(d *Daemon, r *http.Request) Response {
 
 	for _, snap := range existingSnapshots {
 		// Check if an entry for the snapshot already exists in the db.
-		_, snapErr := d.db.ContainerId(snap.Name)
+		_, snapErr := d.cluster.ContainerID(snap.Name)
 		if snapErr != nil {
 			if snapErr != sql.ErrNoRows {
 				return SmartError(snapErr)
@@ -588,10 +655,10 @@ func internalImport(d *Daemon, r *http.Request) Response {
 		}
 
 		// Check if a storage volume entry for the snapshot already exists.
-		_, _, csVolErr := d.db.StoragePoolVolumeGetType(snap.Name,
+		_, _, csVolErr := d.cluster.StoragePoolNodeVolumeGetType(snap.Name,
 			storagePoolVolumeTypeContainer, poolID)
 		if csVolErr != nil {
-			if csVolErr != db.NoSuchObjectError {
+			if csVolErr != db.ErrNoSuchObject {
 				return SmartError(csVolErr)
 			}
 		}
@@ -604,14 +671,14 @@ func internalImport(d *Daemon, r *http.Request) Response {
 		}
 
 		if snapErr == nil {
-			err := d.db.ContainerRemove(snap.Name)
+			err := d.cluster.ContainerRemove(snap.Name)
 			if err != nil {
 				return SmartError(err)
 			}
 		}
 
 		if csVolErr == nil {
-			err := d.db.StoragePoolVolumeDelete(snap.Name,
+			err := d.cluster.StoragePoolVolumeDelete(snap.Name,
 				storagePoolVolumeTypeContainer, poolID)
 			if err != nil {
 				return SmartError(err)
