@@ -11,6 +11,7 @@ import (
 
 	"gopkg.in/lxc/go-lxc.v2"
 
+	"github.com/lxc/lxd/lxd/cluster"
 	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/lxd/sys"
@@ -18,6 +19,7 @@ import (
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/idmap"
+	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/osarch"
 )
 
@@ -102,6 +104,8 @@ func containerValidDeviceConfigKey(t, k string) bool {
 			return true
 		case "path":
 			return true
+		case "required":
+			return true
 		case "uid":
 			return true
 		default:
@@ -218,6 +222,17 @@ func containerValidDeviceConfigKey(t, k string) bool {
 		default:
 			return false
 		}
+	case "proxy":
+		switch k {
+		case "listen":
+			return true
+		case "connect":
+			return true
+		case "bind":
+			return true
+		default:
+			return false
+		}
 	case "none":
 		return false
 	default:
@@ -266,37 +281,7 @@ func containerValidConfig(os *sys.OS, config map[string]string, profile bool, ex
 	return nil
 }
 
-func isRootDiskDevice(device types.Device) bool {
-	if device["type"] == "disk" && device["path"] == "/" && device["source"] == "" {
-		return true
-	}
-
-	return false
-}
-
-func containerGetRootDiskDevice(devices types.Devices) (string, types.Device, error) {
-	var devName string
-	var dev types.Device
-
-	for n, d := range devices {
-		if isRootDiskDevice(d) {
-			if devName != "" {
-				return "", types.Device{}, fmt.Errorf("More than one root device found.")
-			}
-
-			devName = n
-			dev = d
-		}
-	}
-
-	if devName != "" {
-		return devName, dev, nil
-	}
-
-	return "", types.Device{}, fmt.Errorf("No root device could be found.")
-}
-
-func containerValidDevices(db *db.Node, devices types.Devices, profile bool, expanded bool) error {
+func containerValidDevices(db *db.Cluster, devices types.Devices, profile bool, expanded bool) error {
 	// Empty device list
 	if devices == nil {
 		return nil
@@ -309,7 +294,7 @@ func containerValidDevices(db *db.Node, devices types.Devices, profile bool, exp
 			return fmt.Errorf("Missing device type for device '%s'", name)
 		}
 
-		if !shared.StringInSlice(m["type"], []string{"disk", "gpu", "infiniband", "nic", "none", "unix-block", "unix-char", "usb"}) {
+		if !shared.StringInSlice(m["type"], []string{"disk", "gpu", "infiniband", "nic", "none", "proxy", "unix-block", "unix-char", "usb"}) {
 			return fmt.Errorf("Invalid device type for device '%s'", name)
 		}
 
@@ -386,7 +371,7 @@ func containerValidDevices(db *db.Node, devices types.Devices, profile bool, exp
 				return fmt.Errorf("Unix device entry is missing the required \"source\" or \"path\" property.")
 			}
 
-			if m["major"] == "" || m["minor"] == "" {
+			if (m["required"] == "" || shared.IsTrue(m["required"])) && (m["major"] == "" || m["minor"] == "") {
 				srcPath, exist := m["source"]
 				if !exist {
 					srcPath = m["path"]
@@ -415,6 +400,14 @@ func containerValidDevices(db *db.Node, devices types.Devices, profile bool, exp
 		} else if m["type"] == "gpu" {
 			// Probably no checks needed, since we allow users to
 			// pass in all GPUs.
+		} else if m["type"] == "proxy" {
+			if m["listen"] == "" {
+				return fmt.Errorf("Proxy device entry is missing the required \"listen\" property.")
+			}
+
+			if m["connect"] == "" {
+				return fmt.Errorf("Proxy device entry is missing the required \"connect\" property.")
+			}
 		} else if m["type"] == "none" {
 			continue
 		} else {
@@ -424,7 +417,7 @@ func containerValidDevices(db *db.Node, devices types.Devices, profile bool, exp
 
 	// Checks on the expanded config
 	if expanded {
-		_, _, err := containerGetRootDiskDevice(devices)
+		_, _, err := shared.GetRootDiskDevice(devices)
 		if err != nil {
 			return err
 		}
@@ -553,7 +546,7 @@ func containerCreateAsEmpty(d *Daemon, args db.ContainerArgs) (container, error)
 	// Now create the empty storage
 	err = c.Storage().ContainerCreate(c)
 	if err != nil {
-		d.db.ContainerRemove(args.Name)
+		d.cluster.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -577,20 +570,45 @@ func containerCreateEmptySnapshot(s *state.State, args db.ContainerArgs) (contai
 	// Now create the empty snapshot
 	err = c.Storage().ContainerSnapshotCreateEmpty(c)
 	if err != nil {
-		s.DB.ContainerRemove(args.Name)
+		s.Cluster.ContainerRemove(args.Name)
 		return nil, err
 	}
 
 	return c, nil
 }
 
-func containerCreateFromImage(s *state.State, args db.ContainerArgs, hash string) (container, error) {
+func containerCreateFromImage(d *Daemon, args db.ContainerArgs, hash string) (container, error) {
+	s := d.State()
+
 	// Get the image properties
-	_, img, err := s.DB.ImageGet(hash, false, false)
+	_, img, err := s.Cluster.ImageGet(hash, false, false)
 	if err != nil {
 		return nil, err
 	}
 
+	// Check if the image is available locally or it's on another node.
+	nodeAddress, err := s.Cluster.ImageLocate(hash)
+	if err != nil {
+		return nil, err
+	}
+	if nodeAddress != "" {
+		// The image is available from another node, let's try to
+		// import it.
+		logger.Debugf("Transferring image %s from node %s", hash, nodeAddress)
+		client, err := cluster.Connect(nodeAddress, d.endpoints.NetworkCert(), false)
+		if err != nil {
+			return nil, err
+		}
+		err = imageImportFromNode(filepath.Join(d.os.VarDir, "images"), client, hash)
+		if err != nil {
+			return nil, err
+		}
+		err = d.cluster.ImageAssociateNode(hash)
+		if err != nil {
+			return nil, err
+		}
+	}
+
 	// Set the "image.*" keys
 	if img.Properties != nil {
 		for k, v := range img.Properties {
@@ -607,16 +625,16 @@ func containerCreateFromImage(s *state.State, args db.ContainerArgs, hash string
 		return nil, err
 	}
 
-	err = s.DB.ImageLastAccessUpdate(hash, time.Now().UTC())
+	err = s.Cluster.ImageLastAccessUpdate(hash, time.Now().UTC())
 	if err != nil {
-		s.DB.ContainerRemove(args.Name)
+		s.Cluster.ContainerRemove(args.Name)
 		return nil, fmt.Errorf("Error updating image last use date: %s", err)
 	}
 
 	// Now create the storage from an image
 	err = c.Storage().ContainerCreateFromImage(c, hash)
 	if err != nil {
-		s.DB.ContainerRemove(args.Name)
+		s.Cluster.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -641,7 +659,7 @@ func containerCreateAsCopy(s *state.State, args db.ContainerArgs, sourceContaine
 	if !containerOnly {
 		snapshots, err := sourceContainer.Snapshots()
 		if err != nil {
-			s.DB.ContainerRemove(args.Name)
+			s.Cluster.ContainerRemove(args.Name)
 			return nil, err
 		}
 
@@ -673,9 +691,9 @@ func containerCreateAsCopy(s *state.State, args db.ContainerArgs, sourceContaine
 	err = ct.Storage().ContainerCopy(ct, sourceContainer, containerOnly)
 	if err != nil {
 		for _, v := range csList {
-			s.DB.ContainerRemove((*v).Name())
+			s.Cluster.ContainerRemove((*v).Name())
 		}
-		s.DB.ContainerRemove(args.Name)
+		s.Cluster.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -754,7 +772,7 @@ func containerCreateAsSnapshot(s *state.State, args db.ContainerArgs, sourceCont
 	// Clone the container
 	err = sourceContainer.Storage().ContainerSnapshotCreate(c, sourceContainer)
 	if err != nil {
-		s.DB.ContainerRemove(args.Name)
+		s.Cluster.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -777,6 +795,12 @@ func containerCreateAsSnapshot(s *state.State, args db.ContainerArgs, sourceCont
 		os.RemoveAll(sourceContainer.StatePath())
 	}
 
+	eventSendLifecycle("container-snapshot-created",
+		fmt.Sprintf("/1.0/containers/%s", sourceContainer.Name()),
+		map[string]interface{}{
+			"snapshot_name": args.Name,
+		})
+
 	return c, nil
 }
 
@@ -817,7 +841,7 @@ func containerCreateInternal(s *state.State, args db.ContainerArgs) (container,
 	}
 
 	// Validate container devices
-	err = containerValidDevices(s.DB, args.Devices, false, false)
+	err = containerValidDevices(s.Cluster, args.Devices, false, false)
 	if err != nil {
 		return nil, err
 	}
@@ -833,21 +857,28 @@ func containerCreateInternal(s *state.State, args db.ContainerArgs) (container,
 	}
 
 	// Validate profiles
-	profiles, err := s.DB.Profiles()
+	profiles, err := s.Cluster.Profiles()
 	if err != nil {
 		return nil, err
 	}
 
+	checkedProfiles := []string{}
 	for _, profile := range args.Profiles {
 		if !shared.StringInSlice(profile, profiles) {
 			return nil, fmt.Errorf("Requested profile '%s' doesn't exist", profile)
 		}
+
+		if shared.StringInSlice(profile, checkedProfiles) {
+			return nil, fmt.Errorf("Duplicate profile found in request")
+		}
+
+		checkedProfiles = append(checkedProfiles, profile)
 	}
 
 	// Create the container entry
-	id, err := s.DB.ContainerCreate(args)
+	id, err := s.Cluster.ContainerCreate(args)
 	if err != nil {
-		if err == db.DbErrAlreadyDefined {
+		if err == db.ErrAlreadyDefined {
 			thing := "Container"
 			if shared.IsSnapshot(args.Name) {
 				thing = "Snapshot"
@@ -860,12 +891,12 @@ func containerCreateInternal(s *state.State, args db.ContainerArgs) (container,
 	// Wipe any existing log for this container name
 	os.RemoveAll(shared.LogPath(args.Name))
 
-	args.Id = id
+	args.ID = id
 
 	// Read the timestamp from the database
-	dbArgs, err := s.DB.ContainerGet(args.Name)
+	dbArgs, err := s.Cluster.ContainerGet(args.Name)
 	if err != nil {
-		s.DB.ContainerRemove(args.Name)
+		s.Cluster.ContainerRemove(args.Name)
 		return nil, err
 	}
 	args.CreationDate = dbArgs.CreationDate
@@ -874,7 +905,7 @@ func containerCreateInternal(s *state.State, args db.ContainerArgs) (container,
 	// Setup the container struct and finish creation (storage and idmap)
 	c, err := containerLXCCreate(s, args)
 	if err != nil {
-		s.DB.ContainerRemove(args.Name)
+		s.Cluster.ContainerRemove(args.Name)
 		return nil, err
 	}
 
@@ -883,7 +914,7 @@ func containerCreateInternal(s *state.State, args db.ContainerArgs) (container,
 
 func containerConfigureInternal(c container) error {
 	// Find the root device
-	_, rootDiskDevice, err := containerGetRootDiskDevice(c.ExpandedDevices())
+	_, rootDiskDevice, err := shared.GetRootDiskDevice(c.ExpandedDevices())
 	if err != nil {
 		return err
 	}
@@ -929,7 +960,7 @@ func containerConfigureInternal(c container) error {
 
 func containerLoadById(s *state.State, id int) (container, error) {
 	// Get the DB record
-	name, err := s.DB.ContainerName(id)
+	name, err := s.Cluster.ContainerName(id)
 	if err != nil {
 		return nil, err
 	}
@@ -939,7 +970,7 @@ func containerLoadById(s *state.State, id int) (container, error) {
 
 func containerLoadByName(s *state.State, name string) (container, error) {
 	// Get the DB record
-	args, err := s.DB.ContainerGet(name)
+	args, err := s.Cluster.ContainerGet(name)
 	if err != nil {
 		return nil, err
 	}
