@@ -23,6 +23,15 @@ func containerLogsGet(d *Daemon, r *http.Request) Response {
 	 */
 	name := mux.Vars(r)["name"]
 
+	// Handle requests targeted to a container on a different node
+	response, err := ForwardedResponseIfContainerIsRemote(d, r, name)
+	if err != nil {
+		return SmartError(err)
+	}
+	if response != nil {
+		return response
+	}
+
 	if err := containerValidName(name); err != nil {
 		return BadRequest(err)
 	}
@@ -63,6 +72,16 @@ func validLogFileName(fname string) bool {
 
 func containerLogGet(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
+
+	// Handle requests targeted to a container on a different node
+	response, err := ForwardedResponseIfContainerIsRemote(d, r, name)
+	if err != nil {
+		return SmartError(err)
+	}
+	if response != nil {
+		return response
+	}
+
 	file := mux.Vars(r)["file"]
 
 	if err := containerValidName(name); err != nil {
@@ -83,6 +102,16 @@ func containerLogGet(d *Daemon, r *http.Request) Response {
 
 func containerLogDelete(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
+
+	// Handle requests targeted to a container on a different node
+	response, err := ForwardedResponseIfContainerIsRemote(d, r, name)
+	if err != nil {
+		return SmartError(err)
+	}
+	if response != nil {
+		return response
+	}
+
 	file := mux.Vars(r)["file"]
 
 	if err := containerValidName(name); err != nil {
