@@ -23,6 +23,16 @@ import (
 func containerPut(d *Daemon, r *http.Request) Response {
 	// Get the container
 	name := mux.Vars(r)["name"]
+
+	// Handle requests targeted to a container on a different node
+	response, err := ForwardedResponseIfContainerIsRemote(d, r, name)
+	if err != nil {
+		return SmartError(err)
+	}
+	if response != nil {
+		return response
+	}
+
 	c, err := containerLoadByName(d.State(), name)
 	if err != nil {
 		return NotFound
@@ -46,6 +56,7 @@ func containerPut(d *Daemon, r *http.Request) Response {
 	}
 
 	var do func(*operation) error
+	var opDescription string
 	if configRaw.Restore == "" {
 		// Update container configuration
 		do = func(op *operation) error {
@@ -65,17 +76,21 @@ func containerPut(d *Daemon, r *http.Request) Response {
 
 			return nil
 		}
+
+		opDescription = "Updating container"
 	} else {
 		// Snapshot Restore
 		do = func(op *operation) error {
 			return containerSnapRestore(d.State(), name, configRaw.Restore, configRaw.Stateful)
 		}
+
+		opDescription = "Restoring snapshot"
 	}
 
 	resources := map[string][]string{}
 	resources["containers"] = []string{name}
 
-	op, err := operationCreate(operationClassTask, resources, nil, do, nil, nil)
+	op, err := operationCreate(d.cluster, operationClassTask, opDescription, resources, nil, do, nil, nil)
 	if err != nil {
 		return InternalError(err)
 	}
