@@ -6,24 +6,31 @@ import (
 	"fmt"
 	"io/ioutil"
 	"net/http"
-	"strconv"
+	"strings"
 
 	"github.com/gorilla/mux"
 
 	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/version"
 )
 
 func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
-	recursionStr := r.FormValue("recursion")
-	recursion, err := strconv.Atoi(recursionStr)
+	cname := mux.Vars(r)["name"]
+
+	// Handle requests targeted to a container on a different node
+	response, err := ForwardedResponseIfContainerIsRemote(d, r, cname)
 	if err != nil {
-		recursion = 0
+		return SmartError(err)
+	}
+	if response != nil {
+		return response
 	}
 
-	cname := mux.Vars(r)["name"]
+	recursion := util.IsRecursionRequest(r)
+
 	c, err := containerLoadByName(d.State(), cname)
 	if err != nil {
 		return SmartError(err)
@@ -39,7 +46,7 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 
 	for _, snap := range snaps {
 		_, snapName, _ := containerGetParentAndSnapshotName(snap.Name())
-		if recursion == 0 {
+		if !recursion {
 			url := fmt.Sprintf("/%s/containers/%s/snapshots/%s", version.APIVersion, cname, snapName)
 			resultString = append(resultString, url)
 		} else {
@@ -52,7 +59,7 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 		}
 	}
 
-	if recursion == 0 {
+	if !recursion {
 		return SyncResponse(true, resultString)
 	}
 
@@ -62,6 +69,15 @@ func containerSnapshotsGet(d *Daemon, r *http.Request) Response {
 func containerSnapshotsPost(d *Daemon, r *http.Request) Response {
 	name := mux.Vars(r)["name"]
 
+	// Handle requests targeted to a container on a different node
+	response, err := ForwardedResponseIfContainerIsRemote(d, r, name)
+	if err != nil {
+		return SmartError(err)
+	}
+	if response != nil {
+		return response
+	}
+
 	/*
 	 * snapshot is a three step operation:
 	 * 1. choose a new name
@@ -88,10 +104,15 @@ func containerSnapshotsPost(d *Daemon, r *http.Request) Response {
 
 	if req.Name == "" {
 		// come up with a name
-		i := d.db.ContainerNextSnapshot(name)
+		i := d.cluster.ContainerNextSnapshot(name)
 		req.Name = fmt.Sprintf("snap%d", i)
 	}
 
+	// Validate the name
+	if strings.Contains(req.Name, "/") {
+		return BadRequest(fmt.Errorf("Snapshot names may not contain slashes"))
+	}
+
 	fullName := name +
 		shared.SnapshotDelimiter +
 		req.Name
@@ -120,7 +141,7 @@ func containerSnapshotsPost(d *Daemon, r *http.Request) Response {
 	resources := map[string][]string{}
 	resources["containers"] = []string{name}
 
-	op, err := operationCreate(operationClassTask, resources, nil, snapshot, nil, nil)
+	op, err := operationCreate(d.cluster, operationClassTask, "Snapshotting container", resources, nil, snapshot, nil, nil)
 	if err != nil {
 		return InternalError(err)
 	}
@@ -132,6 +153,14 @@ func snapshotHandler(d *Daemon, r *http.Request) Response {
 	containerName := mux.Vars(r)["name"]
 	snapshotName := mux.Vars(r)["snapshotName"]
 
+	response, err := ForwardedResponseIfContainerIsRemote(d, r, containerName)
+	if err != nil {
+		return SmartError(err)
+	}
+	if response != nil {
+		return response
+	}
+
 	sc, err := containerLoadByName(
 		d.State(),
 		containerName+
@@ -217,12 +246,12 @@ func snapshotPost(d *Daemon, r *http.Request, sc container, containerName string
 
 		if req.Target != nil {
 			// Push mode
-			err := ws.ConnectTarget(*req.Target)
+			err := ws.ConnectContainerTarget(*req.Target)
 			if err != nil {
 				return InternalError(err)
 			}
 
-			op, err := operationCreate(operationClassTask, resources, nil, ws.Do, nil, nil)
+			op, err := operationCreate(d.cluster, operationClassTask, "Transferring snapshot", resources, nil, ws.Do, nil, nil)
 			if err != nil {
 				return InternalError(err)
 			}
@@ -231,7 +260,7 @@ func snapshotPost(d *Daemon, r *http.Request, sc container, containerName string
 		}
 
 		// Pull mode
-		op, err := operationCreate(operationClassWebsocket, resources, ws.Metadata(), ws.Do, nil, ws.Connect)
+		op, err := operationCreate(d.cluster, operationClassWebsocket, "Transferring snapshot", resources, ws.Metadata(), ws.Do, nil, ws.Connect)
 		if err != nil {
 			return InternalError(err)
 		}
@@ -244,10 +273,15 @@ func snapshotPost(d *Daemon, r *http.Request, sc container, containerName string
 		return BadRequest(err)
 	}
 
+	// Validate the name
+	if strings.Contains(newName, "/") {
+		return BadRequest(fmt.Errorf("Snapshot names may not contain slashes"))
+	}
+
 	fullName := containerName + shared.SnapshotDelimiter + newName
 
 	// Check that the name isn't already in use
-	id, _ := d.db.ContainerId(fullName)
+	id, _ := d.cluster.ContainerID(fullName)
 	if id > 0 {
 		return Conflict
 	}
@@ -259,7 +293,7 @@ func snapshotPost(d *Daemon, r *http.Request, sc container, containerName string
 	resources := map[string][]string{}
 	resources["containers"] = []string{containerName}
 
-	op, err := operationCreate(operationClassTask, resources, nil, rename, nil, nil)
+	op, err := operationCreate(d.cluster, operationClassTask, "Renaming snapshot", resources, nil, rename, nil, nil)
 	if err != nil {
 		return InternalError(err)
 	}
@@ -275,7 +309,7 @@ func snapshotDelete(sc container, name string) Response {
 	resources := map[string][]string{}
 	resources["containers"] = []string{sc.Name()}
 
-	op, err := operationCreate(operationClassTask, resources, nil, remove, nil, nil)
+	op, err := operationCreate(sc.DaemonState().Cluster, operationClassTask, "Deleting snapshot", resources, nil, remove, nil, nil)
 	if err != nil {
 		return InternalError(err)
 	}
