@@ -4,19 +4,17 @@ import (
 	"database/sql"
 	"fmt"
 
-	_ "github.com/mattn/go-sqlite3"
-
 	"github.com/lxc/lxd/lxd/types"
 	"github.com/lxc/lxd/shared/api"
 )
 
 // Profiles returns a string list of profiles.
-func (n *Node) Profiles() ([]string, error) {
+func (c *Cluster) Profiles() ([]string, error) {
 	q := fmt.Sprintf("SELECT name FROM profiles")
 	inargs := []interface{}{}
 	var name string
 	outfmt := []interface{}{name}
-	result, err := queryScan(n.db, q, inargs, outfmt)
+	result, err := queryScan(c.db, q, inargs, outfmt)
 	if err != nil {
 		return []string{}, err
 	}
@@ -29,24 +27,25 @@ func (n *Node) Profiles() ([]string, error) {
 	return response, nil
 }
 
-func (n *Node) ProfileGet(name string) (int64, *api.Profile, error) {
+// ProfileGet returns the profile with the given name.
+func (c *Cluster) ProfileGet(name string) (int64, *api.Profile, error) {
 	id := int64(-1)
 	description := sql.NullString{}
 
 	q := "SELECT id, description FROM profiles WHERE name=?"
 	arg1 := []interface{}{name}
 	arg2 := []interface{}{&id, &description}
-	err := dbQueryRowScan(n.db, q, arg1, arg2)
+	err := dbQueryRowScan(c.db, q, arg1, arg2)
 	if err != nil {
 		return -1, nil, err
 	}
 
-	config, err := n.ProfileConfig(name)
+	config, err := c.ProfileConfig(name)
 	if err != nil {
 		return -1, nil, err
 	}
 
-	devices, err := n.Devices(name, true)
+	devices, err := c.Devices(name, true)
 	if err != nil {
 		return -1, nil, err
 	}
@@ -62,53 +61,49 @@ func (n *Node) ProfileGet(name string) (int64, *api.Profile, error) {
 	return id, &profile, nil
 }
 
-func (n *Node) ProfileCreate(profile string, description string, config map[string]string,
+// ProfileCreate creates a new profile.
+func (c *Cluster) ProfileCreate(profile string, description string, config map[string]string,
 	devices types.Devices) (int64, error) {
 
-	tx, err := begin(n.db)
-	if err != nil {
-		return -1, err
-	}
-	result, err := tx.Exec("INSERT INTO profiles (name, description) VALUES (?, ?)", profile, description)
-	if err != nil {
-		tx.Rollback()
-		return -1, err
-	}
-	id, err := result.LastInsertId()
-	if err != nil {
-		tx.Rollback()
-		return -1, err
-	}
+	var id int64
+	err := c.Transaction(func(tx *ClusterTx) error {
+		result, err := tx.tx.Exec("INSERT INTO profiles (name, description) VALUES (?, ?)", profile, description)
+		if err != nil {
+			return err
+		}
+		id, err = result.LastInsertId()
+		if err != nil {
+			return err
+		}
 
-	err = ProfileConfigAdd(tx, id, config)
-	if err != nil {
-		tx.Rollback()
-		return -1, err
-	}
+		err = ProfileConfigAdd(tx.tx, id, config)
+		if err != nil {
+			return err
+		}
 
-	err = DevicesAdd(tx, "profile", id, devices)
+		err = DevicesAdd(tx.tx, "profile", id, devices)
+		if err != nil {
+			return err
+		}
+		return nil
+	})
 	if err != nil {
-		tx.Rollback()
-		return -1, err
-	}
-
-	err = TxCommit(tx)
-	if err != nil {
-		return -1, err
+		id = -1
 	}
 
 	return id, nil
 }
 
-func (n *Node) ProfileCreateDefault() error {
-	id, _, _ := n.ProfileGet("default")
+// ProfileCreateDefault creates the default profile.
+func (c *Cluster) ProfileCreateDefault() error {
+	id, _, _ := c.ProfileGet("default")
 
 	if id != -1 {
 		// default profile already exists
 		return nil
 	}
 
-	_, err := n.ProfileCreate("default", "Default LXD profile", map[string]string{}, types.Devices{})
+	_, err := c.ProfileCreate("default", "Default LXD profile", map[string]string{}, types.Devices{})
 	if err != nil {
 		return err
 	}
@@ -116,8 +111,8 @@ func (n *Node) ProfileCreateDefault() error {
 	return nil
 }
 
-// Get the profile configuration map from the DB
-func (n *Node) ProfileConfig(name string) (map[string]string, error) {
+// ProfileConfig gets the profile configuration map from the DB.
+func (c *Cluster) ProfileConfig(name string) (map[string]string, error) {
 	var key, value string
 	query := `
         SELECT
@@ -127,7 +122,7 @@ func (n *Node) ProfileConfig(name string) (map[string]string, error) {
 		WHERE name=?`
 	inargs := []interface{}{name}
 	outfmt := []interface{}{key, value}
-	results, err := queryScan(n.db, query, inargs, outfmt)
+	results, err := queryScan(c.db, query, inargs, outfmt)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to get profile '%s'", name)
 	}
@@ -139,13 +134,13 @@ func (n *Node) ProfileConfig(name string) (map[string]string, error) {
 		 */
 		query := "SELECT id FROM profiles WHERE name=?"
 		var id int
-		results, err := queryScan(n.db, query, []interface{}{name}, []interface{}{id})
+		results, err := queryScan(c.db, query, []interface{}{name}, []interface{}{id})
 		if err != nil {
 			return nil, err
 		}
 
 		if len(results) == 0 {
-			return nil, NoSuchObjectError
+			return nil, ErrNoSuchObject
 		}
 	}
 
@@ -161,13 +156,14 @@ func (n *Node) ProfileConfig(name string) (map[string]string, error) {
 	return config, nil
 }
 
-func (n *Node) ProfileDelete(name string) error {
-	id, _, err := n.ProfileGet(name)
+// ProfileDelete deletes the profile with the given name.
+func (c *Cluster) ProfileDelete(name string) error {
+	id, _, err := c.ProfileGet(name)
 	if err != nil {
 		return err
 	}
 
-	_, err = exec(n.db, "DELETE FROM profiles WHERE id=?", id)
+	err = exec(c.db, "DELETE FROM profiles WHERE id=?", id)
 	if err != nil {
 		return err
 	}
@@ -175,28 +171,22 @@ func (n *Node) ProfileDelete(name string) error {
 	return nil
 }
 
-func (n *Node) ProfileUpdate(name string, newName string) error {
-	tx, err := begin(n.db)
-	if err != nil {
+// ProfileUpdate renames the profile with the given name to the given new name.
+func (c *Cluster) ProfileUpdate(name string, newName string) error {
+	err := c.Transaction(func(tx *ClusterTx) error {
+		_, err := tx.tx.Exec("UPDATE profiles SET name=? WHERE name=?", newName, name)
 		return err
-	}
-
-	_, err = tx.Exec("UPDATE profiles SET name=? WHERE name=?", newName, name)
-	if err != nil {
-		tx.Rollback()
-		return err
-	}
-
-	err = TxCommit(tx)
-
+	})
 	return err
 }
 
+// ProfileDescriptionUpdate updates the description of the profile with the given ID.
 func ProfileDescriptionUpdate(tx *sql.Tx, id int64, description string) error {
 	_, err := tx.Exec("UPDATE profiles SET description=? WHERE id=?", description, id)
 	return err
 }
 
+// ProfileConfigClear resets the config of the profile with the given ID.
 func ProfileConfigClear(tx *sql.Tx, id int64) error {
 	_, err := tx.Exec("DELETE FROM profiles_config WHERE profile_id=?", id)
 	if err != nil {
@@ -218,6 +208,7 @@ func ProfileConfigClear(tx *sql.Tx, id int64) error {
 	return nil
 }
 
+// ProfileConfigAdd adds a config to the profile with the given ID.
 func ProfileConfigAdd(tx *sql.Tx, id int64, config map[string]string) error {
 	str := fmt.Sprintf("INSERT INTO profiles_config (profile_id, key, value) VALUES(?, ?, ?)")
 	stmt, err := tx.Prepare(str)
@@ -236,7 +227,9 @@ func ProfileConfigAdd(tx *sql.Tx, id int64, config map[string]string) error {
 	return nil
 }
 
-func (n *Node) ProfileContainersGet(profile string) ([]string, error) {
+// ProfileContainersGet gets the names of the containers associated with the
+// profile with the given name.
+func (c *Cluster) ProfileContainersGet(profile string) ([]string, error) {
 	q := `SELECT containers.name FROM containers JOIN containers_profiles
 		ON containers.id == containers_profiles.container_id
 		JOIN profiles ON containers_profiles.profile_id == profiles.id
@@ -247,7 +240,7 @@ func (n *Node) ProfileContainersGet(profile string) ([]string, error) {
 	var name string
 	outfmt := []interface{}{name}
 
-	output, err := queryScan(n.db, q, inargs, outfmt)
+	output, err := queryScan(c.db, q, inargs, outfmt)
 	if err != nil {
 		return results, err
 	}
@@ -259,13 +252,14 @@ func (n *Node) ProfileContainersGet(profile string) ([]string, error) {
 	return results, nil
 }
 
-func (n *Node) ProfileCleanupLeftover() error {
+// ProfileCleanupLeftover removes unreferenced profiles.
+func (c *Cluster) ProfileCleanupLeftover() error {
 	stmt := `
 DELETE FROM profiles_config WHERE profile_id NOT IN (SELECT id FROM profiles);
 DELETE FROM profiles_devices WHERE profile_id NOT IN (SELECT id FROM profiles);
 DELETE FROM profiles_devices_config WHERE profile_device_id NOT IN (SELECT id FROM profiles_devices);
 `
-	_, err := n.db.Exec(stmt)
+	err := exec(c.db, stmt)
 	if err != nil {
 		return err
 	}
