@@ -7,10 +7,10 @@ import (
 	"strings"
 )
 
-// Return a parameters expression with the given number of '?'
-// placeholders. E.g. exprParams(2) -> "(?, ?)". Useful for
-// IN expressions.
-func exprParams(n int) string {
+// Params returns a parameters expression with the given number of '?'
+// placeholders. E.g. Params(2) -> "(?, ?)". Useful for IN and VALUES
+// expressions.
+func Params(n int) string {
 	tokens := make([]string, n)
 	for i := 0; i < n; i++ {
 		tokens[i] = "?"
