@@ -2,14 +2,16 @@ package query
 
 import (
 	"database/sql"
-	"fmt"
+
+	"github.com/lxc/lxd/shared/logger"
+	"github.com/pkg/errors"
 )
 
 // Transaction executes the given function within a database transaction.
 func Transaction(db *sql.DB, f func(*sql.Tx) error) error {
 	tx, err := db.Begin()
 	if err != nil {
-		return fmt.Errorf("failed to begin transaction: %v", err)
+		return errors.Wrap(err, "failed to begin transaction")
 	}
 
 	err = f(tx)
@@ -17,7 +19,11 @@ func Transaction(db *sql.DB, f func(*sql.Tx) error) error {
 		return rollback(tx, err)
 	}
 
-	return tx.Commit()
+	err = tx.Commit()
+	if err == sql.ErrTxDone {
+		err = nil // Ignore duplicate commits/rollbacks
+	}
+	return err
 }
 
 // Rollback a transaction after the given error occurred. If the rollback
@@ -26,7 +32,7 @@ func Transaction(db *sql.DB, f func(*sql.Tx) error) error {
 func rollback(tx *sql.Tx, reason error) error {
 	err := tx.Rollback()
 	if err != nil {
-		return fmt.Errorf("failed to rollback transaction after error (%v)", reason)
+		logger.Warnf("failed to rollback transaction after error (%v): %v", reason, err)
 	}
 
 	return reason
