@@ -16,11 +16,11 @@ import (
 	"strconv"
 	"strings"
 	"syscall"
-
-	_ "github.com/mattn/go-sqlite3"
+	"unsafe"
 
 	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/lxd/state"
+	"github.com/lxc/lxd/lxd/sys"
 	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/logger"
@@ -604,7 +604,7 @@ func deviceTaskBalance(s *state.State) {
 	}
 
 	// Iterate through the containers
-	containers, err := s.DB.ContainersList(db.CTypeRegular)
+	containers, err := s.Cluster.ContainersList(db.CTypeRegular)
 	if err != nil {
 		logger.Error("problem loading containers list", log.Ctx{"err": err})
 		return
@@ -730,7 +730,7 @@ func deviceNetworkPriority(s *state.State, netif string) {
 		return
 	}
 
-	containers, err := s.DB.ContainersList(db.CTypeRegular)
+	containers, err := s.Cluster.ContainersList(db.CTypeRegular)
 	if err != nil {
 		return
 	}
@@ -761,7 +761,7 @@ func deviceNetworkPriority(s *state.State, netif string) {
 }
 
 func deviceUSBEvent(s *state.State, usb usbDevice) {
-	containers, err := s.DB.ContainersList(db.CTypeRegular)
+	containers, err := s.Cluster.ContainersList(db.CTypeRegular)
 	if err != nil {
 		logger.Error("problem loading containers list", log.Ctx{"err": err})
 		return
@@ -817,7 +817,7 @@ func deviceUSBEvent(s *state.State, usb usbDevice) {
 func deviceEventListener(s *state.State) {
 	chNetlinkCPU, chNetlinkNetwork, chUSB, err := deviceNetlinkListener()
 	if err != nil {
-		logger.Errorf("scheduler: couldn't setup netlink listener")
+		logger.Errorf("scheduler: couldn't setup netlink listener: %v", err)
 		return
 	}
 
@@ -847,7 +847,7 @@ func deviceEventListener(s *state.State) {
 
 			logger.Debugf("Scheduler: network: %s has been added: updating network priorities", e[0])
 			deviceNetworkPriority(s, e[0])
-			networkAutoAttach(s.DB, e[0])
+			networkAutoAttach(s.Cluster, e[0])
 		case e := <-chUSB:
 			deviceUSBEvent(s, e)
 		case e := <-deviceSchedRebalance:
@@ -1067,40 +1067,6 @@ func deviceParseCPU(cpuAllowance string, cpuPriority string) (string, string, st
 	return fmt.Sprintf("%d", cpuShares), cpuCfsQuota, cpuCfsPeriod, nil
 }
 
-func deviceTotalMemory() (int64, error) {
-	// Open /proc/meminfo
-	f, err := os.Open("/proc/meminfo")
-	if err != nil {
-		return -1, err
-	}
-	defer f.Close()
-
-	// Read it line by line
-	scan := bufio.NewScanner(f)
-	for scan.Scan() {
-		line := scan.Text()
-
-		// We only care about MemTotal
-		if !strings.HasPrefix(line, "MemTotal:") {
-			continue
-		}
-
-		// Extract the before last (value) and last (unit) fields
-		fields := strings.Split(line, " ")
-		value := fields[len(fields)-2] + fields[len(fields)-1]
-
-		// Feed the result to shared.ParseByteSizeString to get an int value
-		valueBytes, err := shared.ParseByteSizeString(value)
-		if err != nil {
-			return -1, err
-		}
-
-		return valueBytes, nil
-	}
-
-	return -1, fmt.Errorf("Couldn't find MemTotal")
-}
-
 func deviceGetParentBlocks(path string) ([]string, error) {
 	var devices []string
 	var device []string
@@ -1593,3 +1559,492 @@ func deviceLoadInfiniband() (map[string]IBF, error) {
 	// check whether the device is an infiniband device
 	return UseableDevices, nil
 }
+
+func deviceInotifyInit(s *state.State) (int, error) {
+	s.OS.InotifyWatch.Lock()
+	defer s.OS.InotifyWatch.Unlock()
+
+	if s.OS.InotifyWatch.Fd >= 0 {
+		return s.OS.InotifyWatch.Fd, nil
+	}
+
+	inFd, err := syscall.InotifyInit1(syscall.IN_CLOEXEC)
+	if err != nil {
+		logger.Errorf("Failed to initialize inotify")
+		return -1, err
+	}
+	logger.Debugf("Initialized inotify with file descriptor %d", inFd)
+
+	s.OS.InotifyWatch.Fd = inFd
+	return inFd, nil
+}
+
+func findClosestLivingAncestor(cleanPath string) (bool, string) {
+	if shared.PathExists(cleanPath) {
+		return true, cleanPath
+	}
+
+	s := cleanPath
+	for {
+		s = filepath.Dir(s)
+		if s == cleanPath {
+			return false, s
+		}
+		if shared.PathExists(s) {
+			return true, s
+		}
+	}
+}
+
+func deviceInotifyAddClosestLivingAncestor(s *state.State, path string) error {
+	cleanPath := filepath.Clean(path)
+	// Find first existing ancestor directory and add it to the target.
+	exists, watchDir := findClosestLivingAncestor(cleanPath)
+	if !exists {
+		return fmt.Errorf("No existing ancestor directory found for \"%s\"", path)
+	}
+
+	err := deviceInotifyAddTarget(s, watchDir)
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+func deviceInotifyAddTarget(s *state.State, path string) error {
+	s.OS.InotifyWatch.Lock()
+	defer s.OS.InotifyWatch.Unlock()
+
+	inFd := s.OS.InotifyWatch.Fd
+	if inFd < 0 {
+		return fmt.Errorf("Inotify instance not intialized")
+	}
+
+	// Do not add the same target twice.
+	_, ok := s.OS.InotifyWatch.Targets[path]
+	if ok {
+		logger.Debugf("Inotify is already watching \"%s\"", path)
+		return nil
+	}
+
+	mask := uint32(0)
+	mask |= syscall.IN_ONLYDIR
+	mask |= syscall.IN_CREATE
+	mask |= syscall.IN_DELETE
+	mask |= syscall.IN_DELETE_SELF
+	wd, err := syscall.InotifyAddWatch(inFd, path, mask)
+	if err != nil {
+		return err
+	}
+
+	s.OS.InotifyWatch.Targets[path] = &sys.InotifyTargetInfo{
+		Mask: mask,
+		Path: path,
+		Wd:   wd,
+	}
+
+	// Add a second key based on the watch file descriptor to the map that
+	// points to the same allocated memory. This is used to reverse engineer
+	// the absolute path when an event happens in the watched directory.
+	// We prefix the key with a \0 character as this is disallowed in
+	// directory and file names and thus guarantees uniqueness of the key.
+	wdString := fmt.Sprintf("\000:%d", wd)
+	s.OS.InotifyWatch.Targets[wdString] = s.OS.InotifyWatch.Targets[path]
+	return nil
+}
+
+func deviceInotifyDel(s *state.State) {
+	s.OS.InotifyWatch.Lock()
+	syscall.Close(s.OS.InotifyWatch.Fd)
+	s.OS.InotifyWatch.Fd = -1
+	s.OS.InotifyWatch.Unlock()
+}
+
+const LXD_BATCH_IN_EVENTS uint = 100
+const LXD_SINGLE_IN_EVENT_SIZE uint = (syscall.SizeofInotifyEvent + syscall.PathMax)
+const LXD_BATCH_IN_BUFSIZE uint = LXD_BATCH_IN_EVENTS * LXD_SINGLE_IN_EVENT_SIZE
+
+func deviceInotifyWatcher(s *state.State) (chan sys.InotifyTargetInfo, error) {
+	targetChan := make(chan sys.InotifyTargetInfo)
+	go func(target chan sys.InotifyTargetInfo) {
+		for {
+			buf := make([]byte, LXD_BATCH_IN_BUFSIZE)
+			n, errno := syscall.Read(s.OS.InotifyWatch.Fd, buf)
+			if errno != nil {
+				if errno == syscall.EINTR {
+					continue
+				}
+
+				deviceInotifyDel(s)
+				return
+			}
+
+			if n < syscall.SizeofInotifyEvent {
+				continue
+			}
+
+			var offset uint32
+			for offset <= uint32(n-syscall.SizeofInotifyEvent) {
+				name := ""
+				event := (*syscall.InotifyEvent)(unsafe.Pointer(&buf[offset]))
+
+				nameLen := uint32(event.Len)
+				if nameLen > 0 {
+					bytes := (*[syscall.PathMax]byte)(unsafe.Pointer(&buf[offset+syscall.SizeofInotifyEvent]))
+					name = strings.TrimRight(string(bytes[0:nameLen]), "\000")
+				}
+
+				target <- sys.InotifyTargetInfo{
+					Mask: uint32(event.Mask),
+					Path: name,
+					Wd:   int(event.Wd),
+				}
+
+				offset += (syscall.SizeofInotifyEvent + nameLen)
+			}
+		}
+	}(targetChan)
+
+	return targetChan, nil
+}
+
+func deviceInotifyDelWatcher(s *state.State, path string) error {
+	s.OS.InotifyWatch.Lock()
+	defer s.OS.InotifyWatch.Unlock()
+
+	if s.OS.InotifyWatch.Fd < 0 {
+		return nil
+	}
+
+	target, ok := s.OS.InotifyWatch.Targets[path]
+	if !ok {
+		logger.Debugf("Inotify target \"%s\" not present", path)
+		return nil
+	}
+
+	ret, err := syscall.InotifyRmWatch(s.OS.InotifyWatch.Fd, uint32(target.Wd))
+	if ret != 0 {
+		// When a file gets deleted the wd for that file will
+		// automatically be deleted from the inotify instance. So
+		// ignore errors here.
+		logger.Debugf("Inotify syscall returned %s for \"%s\"", err, path)
+	}
+	delete(s.OS.InotifyWatch.Targets, path)
+	wdString := fmt.Sprintf("\000:%d", target.Wd)
+	delete(s.OS.InotifyWatch.Targets, wdString)
+	return nil
+}
+
+func createAncestorPaths(cleanPath string) []string {
+	components := strings.Split(cleanPath, "/")
+	ancestors := []string{}
+	newPath := "/"
+	ancestors = append(ancestors, newPath)
+	for _, v := range components[1:] {
+		newPath = filepath.Join(newPath, v)
+		ancestors = append(ancestors, newPath)
+	}
+
+	return ancestors
+}
+
+func deviceInotifyEvent(s *state.State, target *sys.InotifyTargetInfo) {
+	if (target.Mask & syscall.IN_ISDIR) > 0 {
+		if (target.Mask & syscall.IN_CREATE) > 0 {
+			deviceInotifyDirCreateEvent(s, target)
+		} else if (target.Mask & syscall.IN_DELETE) > 0 {
+			deviceInotifyDirDeleteEvent(s, target)
+		}
+		deviceInotifyDirRescan(s)
+	} else if (target.Mask & syscall.IN_DELETE_SELF) > 0 {
+		deviceInotifyDirDeleteEvent(s, target)
+		deviceInotifyDirRescan(s)
+	} else {
+		deviceInotifyFileEvent(s, target)
+	}
+}
+
+func deviceInotifyDirDeleteEvent(s *state.State, target *sys.InotifyTargetInfo) {
+	parentKey := fmt.Sprintf("\000:%d", target.Wd)
+	s.OS.InotifyWatch.RLock()
+	parent, ok := s.OS.InotifyWatch.Targets[parentKey]
+	s.OS.InotifyWatch.RUnlock()
+	if !ok {
+		return
+	}
+
+	// The absolute path of the file for which we received an event?
+	targetName := filepath.Join(parent.Path, target.Path)
+	targetName = filepath.Clean(targetName)
+	err := deviceInotifyDelWatcher(s, targetName)
+	if err != nil {
+		logger.Errorf("Failed to remove \"%s\" from inotify targets: %s", targetName, err)
+	} else {
+		logger.Errorf("Removed \"%s\" from inotify targets", targetName)
+	}
+}
+
+func deviceInotifyDirRescan(s *state.State) {
+	containers, err := s.Cluster.ContainersList(db.CTypeRegular)
+	if err != nil {
+		logger.Errorf("Failed to load containers: %s", err)
+		return
+	}
+
+	for _, name := range containers {
+		containerIf, err := containerLoadByName(s, name)
+		if err != nil {
+			logger.Errorf("Failed to load container \"%s\": %s", name, err)
+			continue
+		}
+
+		c, ok := containerIf.(*containerLXC)
+		if !ok {
+			logger.Errorf("Received device event on non-LXC container")
+			return
+		}
+
+		if !c.IsRunning() {
+			continue
+		}
+
+		devices := c.ExpandedDevices()
+		for _, name := range devices.DeviceNames() {
+			m := devices[name]
+			if !shared.StringInSlice(m["type"], []string{"unix-char", "unix-block"}) {
+				continue
+			}
+
+			if m["required"] == "" || shared.IsTrue(m["required"]) {
+				continue
+			}
+
+			cmp := m["source"]
+			if cmp == "" {
+				cmp = m["path"]
+			}
+			cleanDevPath := filepath.Clean(cmp)
+			if shared.PathExists(cleanDevPath) {
+				c.insertUnixDevice(fmt.Sprintf("unix.%s", name), m)
+			} else {
+				c.removeUnixDevice(fmt.Sprintf("unix.%s", name), m, true)
+			}
+
+			// and add its nearest existing ancestor.
+			err = deviceInotifyAddClosestLivingAncestor(s, cleanDevPath)
+			if err != nil {
+				logger.Errorf("Failed to add \"%s\" to inotify targets: %s", cleanDevPath, err)
+			} else {
+				logger.Debugf("Added \"%s\" to inotify targets", cleanDevPath)
+			}
+		}
+	}
+}
+
+func deviceInotifyDirCreateEvent(s *state.State, target *sys.InotifyTargetInfo) {
+	parentKey := fmt.Sprintf("\000:%d", target.Wd)
+	s.OS.InotifyWatch.RLock()
+	parent, ok := s.OS.InotifyWatch.Targets[parentKey]
+	s.OS.InotifyWatch.RUnlock()
+	if !ok {
+		return
+	}
+
+	containers, err := s.Cluster.ContainersList(db.CTypeRegular)
+	if err != nil {
+		logger.Errorf("Failed to load containers: %s", err)
+		return
+	}
+
+	// The absolute path of the file for which we received an event?
+	targetName := filepath.Join(parent.Path, target.Path)
+	targetName = filepath.Clean(targetName)
+
+	// ancestors
+	del := createAncestorPaths(targetName)
+	keep := []string{}
+	for _, name := range containers {
+		containerIf, err := containerLoadByName(s, name)
+		if err != nil {
+			logger.Errorf("Failed to load container \"%s\": %s", name, err)
+			continue
+		}
+
+		c, ok := containerIf.(*containerLXC)
+		if !ok {
+			logger.Errorf("Received device event on non-LXC container")
+			return
+		}
+
+		if !c.IsRunning() {
+			continue
+		}
+
+		devices := c.ExpandedDevices()
+		for _, name := range devices.DeviceNames() {
+			m := devices[name]
+			if !shared.StringInSlice(m["type"], []string{"unix-char", "unix-block"}) {
+				continue
+			}
+
+			if m["required"] == "" || shared.IsTrue(m["required"]) {
+				continue
+			}
+
+			cmp := m["source"]
+			if cmp == "" {
+				cmp = m["path"]
+			}
+			cleanDevPath := filepath.Clean(cmp)
+
+			for i := len(del) - 1; i >= 0; i-- {
+				// Only keep paths that can be deleted.
+				if strings.HasPrefix(cleanDevPath, del[i]) {
+					if shared.StringInSlice(del[i], keep) {
+						break
+					}
+
+					keep = append(keep, del[i])
+					break
+				}
+			}
+		}
+	}
+
+	for i, v := range del {
+		if shared.StringInSlice(v, keep) {
+			del[i] = ""
+		}
+	}
+
+	for _, v := range del {
+		if v == "" {
+			continue
+		}
+
+		err := deviceInotifyDelWatcher(s, v)
+		if err != nil {
+			logger.Errorf("Failed to remove \"%s\" from inotify targets: %s", v, err)
+		} else {
+			logger.Debugf("Removed \"%s\" from inotify targets", v)
+		}
+	}
+
+	for _, v := range keep {
+		if v == "" {
+			continue
+		}
+
+		err = deviceInotifyAddClosestLivingAncestor(s, v)
+		if err != nil {
+			logger.Errorf("Failed to add \"%s\" to inotify targets: %s", v, err)
+		} else {
+			logger.Debugf("Added \"%s\" to inotify targets", v)
+		}
+	}
+}
+
+func deviceInotifyFileEvent(s *state.State, target *sys.InotifyTargetInfo) {
+	parentKey := fmt.Sprintf("\000:%d", target.Wd)
+	s.OS.InotifyWatch.RLock()
+	parent, ok := s.OS.InotifyWatch.Targets[parentKey]
+	s.OS.InotifyWatch.RUnlock()
+	if !ok {
+		return
+	}
+
+	containers, err := s.Cluster.ContainersList(db.CTypeRegular)
+	if err != nil {
+		logger.Errorf("Failed to load containers: %s", err)
+		return
+	}
+
+	// Does the current file have watchers?
+	hasWatchers := false
+	// The absolute path of the file for which we received an event?
+	targetName := filepath.Join(parent.Path, target.Path)
+	for _, name := range containers {
+		containerIf, err := containerLoadByName(s, name)
+		if err != nil {
+			logger.Errorf("Failed to load container \"%s\": %s", name, err)
+			continue
+		}
+
+		c, ok := containerIf.(*containerLXC)
+		if !ok {
+			logger.Errorf("Received device event on non-LXC container")
+			return
+		}
+
+		if !c.IsRunning() {
+			continue
+		}
+
+		devices := c.ExpandedDevices()
+		for _, name := range devices.DeviceNames() {
+			m := devices[name]
+			if !shared.StringInSlice(m["type"], []string{"unix-char", "unix-block"}) {
+				continue
+			}
+
+			cmp := m["source"]
+			if cmp == "" {
+				cmp = m["path"]
+			}
+
+			if m["required"] == "" || shared.IsTrue(m["required"]) {
+				continue
+			}
+
+			cleanDevPath := filepath.Clean(cmp)
+			cleanInotPath := filepath.Clean(targetName)
+			if !hasWatchers && strings.HasPrefix(cleanDevPath, cleanInotPath) {
+				hasWatchers = true
+			}
+
+			if cleanDevPath != cleanInotPath {
+				continue
+			}
+
+			if (target.Mask & syscall.IN_CREATE) > 0 {
+				err := c.insertUnixDevice(fmt.Sprintf("unix.%s", name), m)
+				if err != nil {
+					logger.Error("Failed to create unix device", log.Ctx{"err": err, "dev": m, "container": c.Name()})
+					continue
+				}
+			} else if (target.Mask & syscall.IN_DELETE) > 0 {
+				err := c.removeUnixDevice(fmt.Sprintf("unix.%s", name), m, true)
+				if err != nil {
+					logger.Error("Failed to remove unix device", log.Ctx{"err": err, "dev": m, "container": c.Name()})
+					continue
+				}
+			} else {
+				logger.Error("Uknown action for unix device", log.Ctx{"dev": m, "container": c.Name()})
+			}
+		}
+	}
+
+	if !hasWatchers {
+		err := deviceInotifyDelWatcher(s, targetName)
+		if err != nil {
+			logger.Errorf("Failed to remove \"%s\" from inotify targets: %s", targetName, err)
+		} else {
+			logger.Debugf("Removed \"%s\" from inotify targets", targetName)
+		}
+	}
+}
+
+func deviceInotifyHandler(s *state.State) {
+	watchChan, err := deviceInotifyWatcher(s)
+	if err != nil {
+		return
+	}
+
+	for {
+		select {
+		case v := <-watchChan:
+			deviceInotifyEvent(s, &v)
+		}
+	}
+}
