@@ -2,38 +2,67 @@ package main
 
 import (
 	"encoding/json"
+	"fmt"
 	"os"
 	"strings"
 	"syscall"
 
+	"github.com/spf13/cobra"
 	"gopkg.in/lxc/go-lxc.v2"
 
 	"github.com/lxc/lxd/shared"
 )
 
-/*
- * This is called by lxd when called as "lxd forkexec <container>"
- */
-func cmdForkExec(args *Args) error {
-	if len(args.Params) < 3 {
-		return SubCommandErrorf(-1, "Bad params: %q", args.Params)
-	}
-	if len(args.Extra) < 1 {
-		return SubCommandErrorf(-1, "Bad extra: %q", args.Extra)
+type cmdForkexec struct {
+	global *cmdGlobal
+}
+
+func (c *cmdForkexec) Command() *cobra.Command {
+	// Main subcommand
+	cmd := &cobra.Command{}
+	cmd.Use = "forkexec <container name> <containers path> <config> -- env [key=value...] -- cmd <args...>"
+	cmd.Short = "Execute a task inside the container"
+	cmd.Long = `Description:
+  Execute a task inside the container
+
+  This internal command is used to spawn a task inside the container and
+  allow LXD to interact with it.
+`
+	cmd.RunE = c.Run
+	cmd.Hidden = true
+
+	return cmd
+}
+
+func (c *cmdForkexec) Run(cmd *cobra.Command, args []string) error {
+	// Sanity checks
+	if len(args) < 4 {
+		cmd.Help()
+
+		if len(args) == 0 {
+			return nil
+		}
+
+		return fmt.Errorf("Missing required arguments")
 	}
 
-	name := args.Params[0]
-	lxcpath := args.Params[1]
-	configPath := args.Params[2]
+	// Only root should run this
+	if os.Geteuid() != 0 {
+		return fmt.Errorf("This must be run as root")
+	}
 
-	c, err := lxc.NewContainer(name, lxcpath)
+	name := args[0]
+	lxcpath := args[1]
+	configPath := args[2]
+
+	d, err := lxc.NewContainer(name, lxcpath)
 	if err != nil {
-		return SubCommandErrorf(-1, "Error initializing container for start: %q", err)
+		return fmt.Errorf("Error initializing container for start: %q", err)
 	}
 
-	err = c.LoadConfigFile(configPath)
+	err = d.LoadConfigFile(configPath)
 	if err != nil {
-		return SubCommandErrorf(-1, "Error opening startup config file: %q", err)
+		return fmt.Errorf("Error opening startup config file: %q", err)
 	}
 
 	syscall.Dup3(int(os.Stdin.Fd()), 200, 0)
@@ -62,10 +91,10 @@ func cmdForkExec(args *Args) error {
 	}
 
 	env := []string{}
-	cmd := []string{}
+	command := []string{}
 
 	section := ""
-	for _, arg := range args.Extra {
+	for _, arg := range args[3:] {
 		// The "cmd" section must come last as it may contain a --
 		if arg == "--" && section != "cmd" {
 			section = ""
@@ -84,17 +113,17 @@ func cmdForkExec(args *Args) error {
 			}
 			env = append(env, arg)
 		} else if section == "cmd" {
-			cmd = append(cmd, arg)
+			command = append(command, arg)
 		} else {
-			return SubCommandErrorf(-1, "Invalid exec section: %s", section)
+			return fmt.Errorf("Invalid exec section: %s", section)
 		}
 	}
 
 	opts.Env = env
 
-	status, err := c.RunCommandNoWait(cmd, opts)
+	status, err := d.RunCommandNoWait(command, opts)
 	if err != nil {
-		return SubCommandErrorf(-1, "Failed running command: %q", err)
+		return fmt.Errorf("Failed running command: %q", err)
 	}
 	// Send the PID of the executing process.
 	w := os.NewFile(uintptr(3), "attachedPid")
@@ -102,23 +131,23 @@ func cmdForkExec(args *Args) error {
 
 	err = json.NewEncoder(w).Encode(status)
 	if err != nil {
-		return SubCommandErrorf(-1, "Failed sending PID of executing command: %q", err)
+		return fmt.Errorf("Failed sending PID of executing command: %q", err)
 	}
 
 	var ws syscall.WaitStatus
 	wpid, err := syscall.Wait4(status, &ws, 0, nil)
 	if err != nil || wpid != status {
-		return SubCommandErrorf(-1, "Failed finding process: %q", err)
+		return fmt.Errorf("Failed finding process: %q", err)
 	}
 
 	if ws.Exited() {
-		return SubCommandErrorf(ws.ExitStatus(), "")
+		os.Exit(ws.ExitStatus())
 	}
 
 	if ws.Signaled() {
 		// 128 + n == Fatal error signal "n"
-		return SubCommandErrorf(128+int(ws.Signal()), "")
+		os.Exit(128 + int(ws.Signal()))
 	}
 
-	return SubCommandErrorf(-1, "Command failed")
+	return fmt.Errorf("Command failed")
 }
