@@ -9,7 +9,6 @@ import (
 	"strings"
 
 	"github.com/gorilla/mux"
-	_ "github.com/mattn/go-sqlite3"
 
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/lxd/util"
@@ -23,7 +22,7 @@ import (
 
 /* This is used for both profiles post and profile put */
 func profilesGet(d *Daemon, r *http.Request) Response {
-	results, err := d.db.Profiles()
+	results, err := d.cluster.Profiles()
 	if err != nil {
 		return SmartError(err)
 	}
@@ -66,7 +65,7 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("No name provided"))
 	}
 
-	_, profile, _ := d.db.ProfileGet(req.Name)
+	_, profile, _ := d.cluster.ProfileGet(req.Name)
 	if profile != nil {
 		return BadRequest(fmt.Errorf("The profile already exists"))
 	}
@@ -84,13 +83,13 @@ func profilesPost(d *Daemon, r *http.Request) Response {
 		return BadRequest(err)
 	}
 
-	err = containerValidDevices(d.db, req.Devices, true, false)
+	err = containerValidDevices(d.cluster, req.Devices, true, false)
 	if err != nil {
 		return BadRequest(err)
 	}
 
 	// Update DB entry
-	_, err = d.db.ProfileCreate(req.Name, req.Description, req.Config, req.Devices)
+	_, err = d.cluster.ProfileCreate(req.Name, req.Description, req.Config, req.Devices)
 	if err != nil {
 		return SmartError(
 			fmt.Errorf("Error inserting %s into database: %s", req.Name, err))
@@ -105,12 +104,12 @@ var profilesCmd = Command{
 	post: profilesPost}
 
 func doProfileGet(s *state.State, name string) (*api.Profile, error) {
-	_, profile, err := s.DB.ProfileGet(name)
+	_, profile, err := s.Cluster.ProfileGet(name)
 	if err != nil {
 		return nil, err
 	}
 
-	cts, err := s.DB.ProfileContainersGet(name)
+	cts, err := s.Cluster.ProfileContainersGet(name)
 	if err != nil {
 		return nil, err
 	}
@@ -139,7 +138,7 @@ func profileGet(d *Daemon, r *http.Request) Response {
 func getContainersWithProfile(s *state.State, profile string) []container {
 	results := []container{}
 
-	output, err := s.DB.ProfileContainersGet(profile)
+	output, err := s.Cluster.ProfileContainersGet(profile)
 	if err != nil {
 		return results
 	}
@@ -159,7 +158,7 @@ func getContainersWithProfile(s *state.State, profile string) []container {
 func profilePut(d *Daemon, r *http.Request) Response {
 	// Get the profile
 	name := mux.Vars(r)["name"]
-	id, profile, err := d.db.ProfileGet(name)
+	id, profile, err := d.cluster.ProfileGet(name)
 	if err != nil {
 		return SmartError(fmt.Errorf("Failed to retrieve profile='%s'", name))
 	}
@@ -176,13 +175,13 @@ func profilePut(d *Daemon, r *http.Request) Response {
 		return BadRequest(err)
 	}
 
-	return doProfileUpdate(d, name, id, profile, req)
+	return SmartError(doProfileUpdate(d, name, id, profile, req))
 }
 
 func profilePatch(d *Daemon, r *http.Request) Response {
 	// Get the profile
 	name := mux.Vars(r)["name"]
-	id, profile, err := d.db.ProfileGet(name)
+	id, profile, err := d.cluster.ProfileGet(name)
 	if err != nil {
 		return SmartError(fmt.Errorf("Failed to retrieve profile='%s'", name))
 	}
@@ -242,7 +241,7 @@ func profilePatch(d *Daemon, r *http.Request) Response {
 		}
 	}
 
-	return doProfileUpdate(d, name, id, profile, req)
+	return SmartError(doProfileUpdate(d, name, id, profile, req))
 }
 
 // The handler for the post operation.
@@ -260,7 +259,7 @@ func profilePost(d *Daemon, r *http.Request) Response {
 	}
 
 	// Check that the name isn't already in use
-	id, _, _ := d.db.ProfileGet(req.Name)
+	id, _, _ := d.cluster.ProfileGet(req.Name)
 	if id > 0 {
 		return Conflict
 	}
@@ -273,7 +272,7 @@ func profilePost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("Invalid profile name '%s'", req.Name))
 	}
 
-	err := d.db.ProfileUpdate(name, req.Name)
+	err := d.cluster.ProfileUpdate(name, req.Name)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -295,7 +294,7 @@ func profileDelete(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("Profile is currently in use"))
 	}
 
-	err = d.db.ProfileDelete(name)
+	err = d.cluster.ProfileDelete(name)
 	if err != nil {
 		return SmartError(err)
 	}
