@@ -12,12 +12,14 @@ import (
 	"github.com/gorilla/websocket"
 
 	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/migration"
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/idmap"
 	"github.com/lxc/lxd/shared/ioprogress"
 	"github.com/lxc/lxd/shared/logger"
+	"github.com/lxc/lxd/shared/version"
 )
 
 // lxdStorageLockMap is a hashmap that allows functions to check whether the
@@ -135,6 +137,7 @@ type storage interface {
 	GetStorageType() storageType
 	GetStorageTypeName() string
 	GetStorageTypeVersion() string
+	GetState() *state.State
 
 	// Functions dealing with storage pools.
 	StoragePoolInit() error
@@ -147,6 +150,7 @@ type storage interface {
 	StoragePoolUpdate(writable *api.StoragePoolPut, changedConfig []string) error
 	GetStoragePoolWritable() api.StoragePoolPut
 	SetStoragePoolWritable(writable *api.StoragePoolPut)
+	GetStoragePool() *api.StoragePool
 
 	// Functions dealing with custom storage volumes.
 	StoragePoolVolumeCreate() error
@@ -155,8 +159,10 @@ type storage interface {
 	StoragePoolVolumeUmount() (bool, error)
 	StoragePoolVolumeUpdate(writable *api.StorageVolumePut, changedConfig []string) error
 	StoragePoolVolumeRename(newName string) error
+	StoragePoolVolumeCopy(source *api.StorageVolumeSource) error
 	GetStoragePoolVolumeWritable() api.StorageVolumePut
 	SetStoragePoolVolumeWritable(writable *api.StorageVolumePut)
+	GetStoragePoolVolume() *api.StorageVolume
 
 	// Functions dealing with container storage volumes.
 	// ContainerCreate creates an empty container (no rootfs/metadata.yaml)
@@ -194,7 +200,7 @@ type storage interface {
 	StorageEntitySetQuota(volumeType int, size int64, data interface{}) error
 
 	// Functions dealing with migration.
-	MigrationType() MigrationFSType
+	MigrationType() migration.MigrationFSType
 	// Does this storage backend preserve inodes when it is moved across LXD
 	// hosts?
 	PreservesInodes() bool
@@ -220,11 +226,14 @@ type storage interface {
 	MigrationSink(
 		live bool,
 		c container,
-		objects []*Snapshot,
+		objects []*migration.Snapshot,
 		conn *websocket.Conn,
 		srcIdmap *idmap.IdmapSet,
 		op *operation,
 		containerOnly bool) error
+
+	StorageMigrationSource() (MigrationStorageSourceDriver, error)
+	StorageMigrationSink(conn *websocket.Conn, op *operation, storage storage) error
 }
 
 func storageCoreInit(driver string) (storage, error) {
@@ -283,7 +292,7 @@ func storageCoreInit(driver string) (storage, error) {
 
 func storageInit(s *state.State, poolName string, volumeName string, volumeType int) (storage, error) {
 	// Load the storage pool.
-	poolID, pool, err := s.DB.StoragePoolGet(poolName)
+	poolID, pool, err := s.Cluster.StoragePoolGet(poolName)
 	if err != nil {
 		return nil, err
 	}
@@ -298,7 +307,7 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 	// Load the storage volume.
 	volume := &api.StorageVolume{}
 	if volumeName != "" && volumeType >= 0 {
-		_, volume, err = s.DB.StoragePoolVolumeGetType(volumeName, volumeType, poolID)
+		_, volume, err = s.Cluster.StoragePoolNodeVolumeGetType(volumeName, volumeType, poolID)
 		if err != nil {
 			return nil, err
 		}
@@ -316,7 +325,6 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		btrfs.pool = pool
 		btrfs.volume = volume
 		btrfs.s = s
-		btrfs.db = s.DB
 		err = btrfs.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -328,7 +336,6 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		dir.pool = pool
 		dir.volume = volume
 		dir.s = s
-		dir.db = s.DB
 		err = dir.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -340,7 +347,6 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		ceph.pool = pool
 		ceph.volume = volume
 		ceph.s = s
-		ceph.db = s.DB
 		err = ceph.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -352,7 +358,6 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		lvm.pool = pool
 		lvm.volume = volume
 		lvm.s = s
-		lvm.db = s.DB
 		err = lvm.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -364,7 +369,6 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		mock.pool = pool
 		mock.volume = volume
 		mock.s = s
-		mock.db = s.DB
 		err = mock.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -376,7 +380,6 @@ func storageInit(s *state.State, poolName string, volumeName string, volumeType
 		zfs.pool = pool
 		zfs.volume = volume
 		zfs.s = s
-		zfs.db = s.DB
 		err = zfs.StoragePoolInit()
 		if err != nil {
 			return nil, err
@@ -517,11 +520,11 @@ func storagePoolVolumeAttachInit(s *state.State, poolName string, volumeName str
 
 	st.SetStoragePoolVolumeWritable(&poolVolumePut)
 
-	poolID, err := s.DB.StoragePoolGetID(poolName)
+	poolID, err := s.Cluster.StoragePoolGetID(poolName)
 	if err != nil {
 		return nil, err
 	}
-	err = s.DB.StoragePoolVolumeUpdate(volumeName, volumeType, poolID, poolVolumePut.Description, poolVolumePut.Config)
+	err = s.Cluster.StoragePoolVolumeUpdate(volumeName, volumeType, poolID, poolVolumePut.Description, poolVolumePut.Config)
 	if err != nil {
 		return nil, err
 	}
@@ -544,7 +547,7 @@ func storagePoolVolumeContainerCreateInit(s *state.State, poolName string, conta
 
 func storagePoolVolumeContainerLoadInit(s *state.State, containerName string) (storage, error) {
 	// Get the storage pool of a given container.
-	poolName, err := s.DB.ContainerPool(containerName)
+	poolName, err := s.Cluster.ContainerPool(containerName)
 	if err != nil {
 		return nil, err
 	}
@@ -810,9 +813,9 @@ func StorageProgressWriter(op *operation, key string, description string) func(i
 }
 
 func SetupStorageDriver(s *state.State, forceCheck bool) error {
-	pools, err := s.DB.StoragePools()
+	pools, err := s.Cluster.StoragePoolsNotPending()
 	if err != nil {
-		if err == db.NoSuchObjectError {
+		if err == db.ErrNoSuchObject {
 			logger.Debugf("No existing storage pools detected.")
 			return nil
 		}
@@ -827,7 +830,7 @@ func SetupStorageDriver(s *state.State, forceCheck bool) error {
 	// but the upgrade somehow got messed up then there will be no
 	// "storage_api" entry in the db.
 	if len(pools) > 0 && !forceCheck {
-		appliedPatches, err := s.DB.Patches()
+		appliedPatches, err := s.Node.Patches()
 		if err != nil {
 			return err
 		}
@@ -854,11 +857,11 @@ func SetupStorageDriver(s *state.State, forceCheck bool) error {
 	}
 
 	// Update the storage drivers cache in api_1.0.go.
-	storagePoolDriversCacheUpdate(s.DB)
+	storagePoolDriversCacheUpdate(s.Cluster)
 	return nil
 }
 
-func storagePoolDriversCacheUpdate(dbNode *db.Node) {
+func storagePoolDriversCacheUpdate(cluster *db.Cluster) {
 	// Get a list of all storage drivers currently in use
 	// on this LXD instance. Only do this when we do not already have done
 	// this once to avoid unnecessarily querying the db. All subsequent
@@ -869,8 +872,8 @@ func storagePoolDriversCacheUpdate(dbNode *db.Node) {
 	// appropriate. (Should be cheaper then querying the db all the time,
 	// especially if we keep adding more storage drivers.)
 
-	drivers, err := dbNode.StoragePoolsGetDrivers()
-	if err != nil && err != db.NoSuchObjectError {
+	drivers, err := cluster.StoragePoolsGetDrivers()
+	if err != nil && err != db.ErrNoSuchObject {
 		return
 	}
 
@@ -886,6 +889,14 @@ func storagePoolDriversCacheUpdate(dbNode *db.Node) {
 		data[driver] = sCore.GetStorageTypeVersion()
 	}
 
+	backends := []string{}
+	for k, v := range data {
+		backends = append(backends, fmt.Sprintf("%s %s", k, v))
+	}
+
+	// Update the agent
+	version.UserAgentStorageBackends(backends)
+
 	storagePoolDriversCacheLock.Lock()
 	storagePoolDriversCacheVal.Store(data)
 	storagePoolDriversCacheLock.Unlock()
