@@ -8,6 +8,7 @@ import (
 	"github.com/gorilla/websocket"
 
 	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/migration"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/idmap"
 	"github.com/lxc/lxd/shared/logger"
@@ -169,8 +170,8 @@ func (s *rbdMigrationSourceDriver) SendWhileRunning(conn *websocket.Conn,
 	return nil
 }
 
-func (s *storageCeph) MigrationType() MigrationFSType {
-	return MigrationFSType_RBD
+func (s *storageCeph) MigrationType() migration.MigrationFSType {
+	return migration.MigrationFSType_RBD
 }
 
 func (s *storageCeph) PreservesInodes() bool {
@@ -209,7 +210,7 @@ func (s *storageCeph) MigrationSource(c container, containerOnly bool) (Migratio
 		s.OSDPoolName, containerName,
 		storagePoolVolumeTypeNameContainer, s.UserName)
 	if err != nil {
-		if err != db.NoSuchObjectError {
+		if err != db.ErrNoSuchObject {
 			logger.Errorf(`Failed to list snapshots for RBD storage volume "%s" on storage pool "%s": %s`, containerName, s.pool.Name, err)
 			return nil, err
 		}
@@ -240,13 +241,13 @@ func (s *storageCeph) MigrationSource(c container, containerOnly bool) (Migratio
 }
 
 func (s *storageCeph) MigrationSink(live bool, c container,
-	snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet,
+	snapshots []*migration.Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet,
 	op *operation, containerOnly bool) error {
 	// Check that we received a valid root disk device with a pool property
 	// set.
 	parentStoragePool := ""
 	parentExpandedDevices := c.ExpandedDevices()
-	parentLocalRootDiskDeviceKey, parentLocalRootDiskDevice, _ := containerGetRootDiskDevice(parentExpandedDevices)
+	parentLocalRootDiskDeviceKey, parentLocalRootDiskDevice, _ := shared.GetRootDiskDevice(parentExpandedDevices)
 	if parentLocalRootDiskDeviceKey != "" {
 		parentStoragePool = parentLocalRootDiskDevice["pool"]
 	}
@@ -312,7 +313,7 @@ func (s *storageCeph) MigrationSink(live bool, c container,
 		// disk device for the snapshot comes from a profile on the new
 		// instance as well we don't need to do anything.
 		if args.Devices != nil {
-			snapLocalRootDiskDeviceKey, _, _ := containerGetRootDiskDevice(args.Devices)
+			snapLocalRootDiskDeviceKey, _, _ := shared.GetRootDiskDevice(args.Devices)
 			if snapLocalRootDiskDeviceKey != "" {
 				args.Devices[snapLocalRootDiskDeviceKey]["pool"] = parentStoragePool
 			}
