@@ -2,13 +2,16 @@ package main
 
 import (
 	"fmt"
+	"time"
 
 	"github.com/gorilla/websocket"
 
 	"github.com/lxc/lxd/lxd/db"
+	"github.com/lxc/lxd/lxd/migration"
 	"github.com/lxc/lxd/lxd/types"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/idmap"
+	"github.com/lxc/lxd/shared/logger"
 )
 
 // MigrationStorageSourceDriver defines the functions needed to implement a
@@ -33,6 +36,8 @@ type MigrationStorageSourceDriver interface {
 	 * to clean up any temporary snapshots, etc.
 	 */
 	Cleanup()
+
+	SendStorageVolume(conn *websocket.Conn, op *operation, bwlimit string, storage storage) error
 }
 
 type rsyncStorageSourceDriver struct {
@@ -44,6 +49,26 @@ func (s rsyncStorageSourceDriver) Snapshots() []container {
 	return s.snapshots
 }
 
+func (s rsyncStorageSourceDriver) SendStorageVolume(conn *websocket.Conn, op *operation, bwlimit string, storage storage) error {
+	ourMount, err := storage.StoragePoolVolumeMount()
+	if err != nil {
+		return err
+	}
+	if ourMount {
+		defer storage.StoragePoolVolumeUmount()
+	}
+
+	pool := storage.GetStoragePool()
+	volume := storage.GetStoragePoolVolume()
+
+	wrapper := StorageProgressReader(op, "fs_progress", volume.Name)
+	state := storage.GetState()
+	path := getStoragePoolVolumeMountPoint(pool.Name, volume.Name)
+	path = shared.AddSlash(path)
+	logger.Debugf("Starting to send storage volume %s on storage pool %s from %s", volume.Name, pool.Name, path)
+	return RsyncSend(volume.Name, path, conn, wrapper, bwlimit, state.OS.ExecPath)
+}
+
 func (s rsyncStorageSourceDriver) SendWhileRunning(conn *websocket.Conn, op *operation, bwlimit string, containerOnly bool) error {
 	ctName, _, _ := containerGetParentAndSnapshotName(s.container.Name())
 
@@ -83,6 +108,10 @@ func (s rsyncStorageSourceDriver) Cleanup() {
 	// noop
 }
 
+func rsyncStorageMigrationSource() (MigrationStorageSourceDriver, error) {
+	return rsyncStorageSourceDriver{nil, nil}, nil
+}
+
 func rsyncMigrationSource(c container, containerOnly bool) (MigrationStorageSourceDriver, error) {
 	var err error
 	var snapshots = []container{}
@@ -96,7 +125,7 @@ func rsyncMigrationSource(c container, containerOnly bool) (MigrationStorageSour
 	return rsyncStorageSourceDriver{c, snapshots}, nil
 }
 
-func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) db.ContainerArgs {
+func snapshotProtobufToContainerArgs(containerName string, snap *migration.Snapshot) db.ContainerArgs {
 	config := map[string]string{}
 
 	for _, ent := range snap.LocalConfig {
@@ -114,7 +143,7 @@ func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) db.Co
 	}
 
 	name := containerName + shared.SnapshotDelimiter + snap.GetName()
-	return db.ContainerArgs{
+	args := db.ContainerArgs{
 		Name:         name,
 		Ctype:        db.CTypeSnapshot,
 		Config:       config,
@@ -124,9 +153,43 @@ func snapshotProtobufToContainerArgs(containerName string, snap *Snapshot) db.Co
 		Architecture: int(snap.GetArchitecture()),
 		Stateful:     snap.GetStateful(),
 	}
+
+	if snap.GetCreationDate() != 0 {
+		args.CreationDate = time.Unix(snap.GetCreationDate(), 0)
+	}
+
+	if snap.GetLastUsedDate() != 0 {
+		args.LastUsedDate = time.Unix(snap.GetLastUsedDate(), 0)
+	}
+
+	return args
 }
 
-func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet, op *operation, containerOnly bool) error {
+func rsyncStorageMigrationSink(conn *websocket.Conn, op *operation, storage storage) error {
+	err := storage.StoragePoolVolumeCreate()
+	if err != nil {
+		return err
+	}
+
+	ourMount, err := storage.StoragePoolVolumeMount()
+	if err != nil {
+		return err
+	}
+	if ourMount {
+		defer storage.StoragePoolVolumeUmount()
+	}
+
+	pool := storage.GetStoragePool()
+	volume := storage.GetStoragePoolVolume()
+
+	wrapper := StorageProgressWriter(op, "fs_progress", volume.Name)
+	path := getStoragePoolVolumeMountPoint(pool.Name, volume.Name)
+	path = shared.AddSlash(path)
+	logger.Debugf("Starting to receive storage volume %s on storage pool %s into %s", volume.Name, pool.Name, path)
+	return RsyncRecv(path, conn, wrapper)
+}
+
+func rsyncMigrationSink(live bool, container container, snapshots []*migration.Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet, op *operation, containerOnly bool) error {
 	ourStart, err := container.StorageStart()
 	if err != nil {
 		return err
@@ -139,7 +202,7 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 	// disk device so we can simply retrieve it from the expanded devices.
 	parentStoragePool := ""
 	parentExpandedDevices := container.ExpandedDevices()
-	parentLocalRootDiskDeviceKey, parentLocalRootDiskDevice, _ := containerGetRootDiskDevice(parentExpandedDevices)
+	parentLocalRootDiskDeviceKey, parentLocalRootDiskDevice, _ := shared.GetRootDiskDevice(parentExpandedDevices)
 	if parentLocalRootDiskDeviceKey != "" {
 		parentStoragePool = parentLocalRootDiskDevice["pool"]
 	}
@@ -161,7 +224,7 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 				// profile on the new instance as well we don't need to
 				// do anything.
 				if args.Devices != nil {
-					snapLocalRootDiskDeviceKey, _, _ := containerGetRootDiskDevice(args.Devices)
+					snapLocalRootDiskDeviceKey, _, _ := shared.GetRootDiskDevice(args.Devices)
 					if snapLocalRootDiskDeviceKey != "" {
 						args.Devices[snapLocalRootDiskDeviceKey]["pool"] = parentStoragePool
 					}
@@ -200,7 +263,7 @@ func rsyncMigrationSink(live bool, container container, snapshots []*Snapshot, c
 				// profile on the new instance as well we don't need to
 				// do anything.
 				if args.Devices != nil {
-					snapLocalRootDiskDeviceKey, _, _ := containerGetRootDiskDevice(args.Devices)
+					snapLocalRootDiskDeviceKey, _, _ := shared.GetRootDiskDevice(args.Devices)
 					if snapLocalRootDiskDeviceKey != "" {
 						args.Devices[snapLocalRootDiskDeviceKey]["pool"] = parentStoragePool
 					}
