@@ -5,6 +5,8 @@ import (
 
 	"github.com/gorilla/websocket"
 
+	"github.com/lxc/lxd/lxd/migration"
+	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/idmap"
 	"github.com/lxc/lxd/shared/logger"
@@ -204,8 +206,8 @@ func (s *storageMock) ImageUmount(fingerprint string) (bool, error) {
 	return true, nil
 }
 
-func (s *storageMock) MigrationType() MigrationFSType {
-	return MigrationFSType_RSYNC
+func (s *storageMock) MigrationType() migration.MigrationFSType {
+	return migration.MigrationFSType_RSYNC
 }
 
 func (s *storageMock) PreservesInodes() bool {
@@ -215,7 +217,8 @@ func (s *storageMock) PreservesInodes() bool {
 func (s *storageMock) MigrationSource(container container, containerOnly bool) (MigrationStorageSourceDriver, error) {
 	return nil, fmt.Errorf("not implemented")
 }
-func (s *storageMock) MigrationSink(live bool, container container, snapshots []*Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet, op *operation, containerOnly bool) error {
+
+func (s *storageMock) MigrationSink(live bool, container container, snapshots []*migration.Snapshot, conn *websocket.Conn, srcIdmap *idmap.IdmapSet, op *operation, containerOnly bool) error {
 	return nil
 }
 
@@ -226,3 +229,27 @@ func (s *storageMock) StorageEntitySetQuota(volumeType int, size int64, data int
 func (s *storageMock) StoragePoolResources() (*api.ResourcesStoragePool, error) {
 	return &api.ResourcesStoragePool{}, nil
 }
+
+func (s *storageMock) StoragePoolVolumeCopy(source *api.StorageVolumeSource) error {
+	return nil
+}
+
+func (s *storageMock) StorageMigrationSource() (MigrationStorageSourceDriver, error) {
+	return nil, nil
+}
+
+func (s *storageMock) StorageMigrationSink(conn *websocket.Conn, op *operation, storage storage) error {
+	return nil
+}
+
+func (s *storageMock) GetStoragePool() *api.StoragePool {
+	return nil
+}
+
+func (s *storageMock) GetStoragePoolVolume() *api.StorageVolume {
+	return nil
+}
+
+func (s *storageMock) GetState() *state.State {
+	return nil
+}
