@@ -1,17 +1,24 @@
 package main
 
 import (
+	"bytes"
+	"crypto/x509"
 	"encoding/json"
+	"encoding/pem"
 	"fmt"
 	"net/http"
-	"strconv"
+	"strings"
 
 	"github.com/gorilla/mux"
+	"github.com/gorilla/websocket"
 	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/lxd/util"
 	"github.com/lxc/lxd/shared"
 	"github.com/lxc/lxd/shared/api"
+	"github.com/lxc/lxd/shared/logger"
 	"github.com/lxc/lxd/shared/version"
+
+	log "github.com/lxc/lxd/shared/log15"
 )
 
 // /1.0/storage-pools/{name}/volumes
@@ -19,23 +26,19 @@ import (
 func storagePoolVolumesGet(d *Daemon, r *http.Request) Response {
 	poolName := mux.Vars(r)["name"]
 
-	recursionStr := r.FormValue("recursion")
-	recursion, err := strconv.Atoi(recursionStr)
-	if err != nil {
-		recursion = 0
-	}
+	recursion := util.IsRecursionRequest(r)
 
 	// Retrieve ID of the storage pool (and check if the storage pool
 	// exists).
-	poolID, err := d.db.StoragePoolGetID(poolName)
+	poolID, err := d.cluster.StoragePoolGetID(poolName)
 	if err != nil {
 		return SmartError(err)
 	}
 
 	// Get all volumes currently attached to the storage pool by ID of the
 	// pool.
-	volumes, err := d.db.StoragePoolVolumesGet(poolID, supportedVolumeTypes)
-	if err != nil && err != db.NoSuchObjectError {
+	volumes, err := d.cluster.StoragePoolVolumesGet(poolID, supportedVolumeTypes)
+	if err != nil && err != db.ErrNoSuchObject {
 		return SmartError(err)
 	}
 
@@ -46,7 +49,7 @@ func storagePoolVolumesGet(d *Daemon, r *http.Request) Response {
 			return InternalError(err)
 		}
 
-		if recursion == 0 {
+		if !recursion {
 			resultString = append(resultString, fmt.Sprintf("/%s/storage-pools/%s/volumes/%s/%s", version.APIVersion, poolName, apiEndpoint, volume.Name))
 		} else {
 			volumeUsedBy, err := storagePoolVolumeUsedByGet(d.State(), volume.Name, volume.Type)
@@ -57,7 +60,7 @@ func storagePoolVolumesGet(d *Daemon, r *http.Request) Response {
 		}
 	}
 
-	if recursion == 0 {
+	if !recursion {
 		return SyncResponse(true, resultString)
 	}
 
@@ -73,11 +76,7 @@ func storagePoolVolumesTypeGet(d *Daemon, r *http.Request) Response {
 	// attached to.
 	poolName := mux.Vars(r)["name"]
 
-	recursionStr := r.FormValue("recursion")
-	recursion, err := strconv.Atoi(recursionStr)
-	if err != nil {
-		recursion = 0
-	}
+	recursion := util.IsRecursionRequest(r)
 
 	// Get the name of the volume type.
 	volumeTypeName := mux.Vars(r)["type"]
@@ -94,14 +93,14 @@ func storagePoolVolumesTypeGet(d *Daemon, r *http.Request) Response {
 
 	// Retrieve ID of the storage pool (and check if the storage pool
 	// exists).
-	poolID, err := d.db.StoragePoolGetID(poolName)
+	poolID, err := d.cluster.StoragePoolGetID(poolName)
 	if err != nil {
 		return SmartError(err)
 	}
 
 	// Get the names of all storage volumes of a given volume type currently
 	// attached to the storage pool.
-	volumes, err := d.db.StoragePoolVolumesGetType(volumeType, poolID)
+	volumes, err := d.cluster.StoragePoolNodeVolumesGetType(volumeType, poolID)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -109,14 +108,14 @@ func storagePoolVolumesTypeGet(d *Daemon, r *http.Request) Response {
 	resultString := []string{}
 	resultMap := []*api.StorageVolume{}
 	for _, volume := range volumes {
-		if recursion == 0 {
+		if !recursion {
 			apiEndpoint, err := storagePoolVolumeTypeToAPIEndpoint(volumeType)
 			if err != nil {
 				return InternalError(err)
 			}
 			resultString = append(resultString, fmt.Sprintf("/%s/storage-pools/%s/volumes/%s/%s", version.APIVersion, poolName, apiEndpoint, volume))
 		} else {
-			_, vol, err := d.db.StoragePoolVolumeGetType(volume, volumeType, poolID)
+			_, vol, err := d.cluster.StoragePoolNodeVolumeGetType(volume, volumeType, poolID)
 			if err != nil {
 				continue
 			}
@@ -131,7 +130,7 @@ func storagePoolVolumesTypeGet(d *Daemon, r *http.Request) Response {
 		}
 	}
 
-	if recursion == 0 {
+	if !recursion {
 		return SyncResponse(true, resultString)
 	}
 
@@ -141,6 +140,11 @@ func storagePoolVolumesTypeGet(d *Daemon, r *http.Request) Response {
 // /1.0/storage-pools/{name}/volumes/{type}
 // Create a storage volume of a given volume type in a given storage pool.
 func storagePoolVolumesTypePost(d *Daemon, r *http.Request) Response {
+	response := ForwardedResponseIfTargetIsRemote(d, r)
+	if response != nil {
+		return response
+	}
+
 	req := api.StorageVolumesPost{}
 
 	// Parse the request.
@@ -154,16 +158,16 @@ func storagePoolVolumesTypePost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("No name provided"))
 	}
 
+	if strings.Contains(req.Name, "/") {
+		return BadRequest(fmt.Errorf("Storage volume names may not contain slashes"))
+	}
+
 	// Check that the user gave use a storage volume type for the storage
 	// volume we are about to create.
 	if req.Type == "" {
 		return BadRequest(fmt.Errorf("you must provide a storage volume type of the storage volume"))
 	}
 
-	// Check if the user gave us a valid pool name in which the new storage
-	// volume is supposed to be created.
-	poolName := mux.Vars(r)["name"]
-
 	// We currently only allow to create storage volumes of type
 	// storagePoolVolumeTypeCustom. So check, that nothing else was
 	// requested.
@@ -172,17 +176,128 @@ func storagePoolVolumesTypePost(d *Daemon, r *http.Request) Response {
 			`storage volumes of type %s`, req.Type))
 	}
 
-	err = storagePoolVolumeCreateInternal(d.State(), poolName, req.Name, req.Description, req.Type, req.Config)
+	poolName := mux.Vars(r)["name"]
+
+	switch req.Source.Type {
+	case "":
+		return doVolumeCreateOrCopy(d, poolName, &req)
+	case "copy":
+		return doVolumeCreateOrCopy(d, poolName, &req)
+	case "migration":
+		return doVolumeMigration(d, poolName, &req)
+	default:
+		return BadRequest(fmt.Errorf("unknown source type %s", req.Source.Type))
+	}
+}
+
+func doVolumeCreateOrCopy(d *Daemon, poolName string, req *api.StorageVolumesPost) Response {
+	doWork := func() error {
+		err := storagePoolVolumeCreateInternal(d.State(), poolName, req)
+		if err != nil {
+			return err
+		}
+		return nil
+	}
+
+	if req.Source.Name == "" {
+		err := doWork()
+		if err != nil {
+			return SmartError(err)
+		}
+
+		return EmptySyncResponse
+	}
+
+	run := func(op *operation) error {
+		return doWork()
+	}
+
+	op, err := operationCreate(d.cluster, operationClassTask, "Copying storage volume", nil, nil, run, nil, nil)
 	if err != nil {
 		return InternalError(err)
 	}
 
-	apiEndpoint, err := storagePoolVolumeTypeNameToAPIEndpoint(req.Type)
+	return OperationResponse(op)
+}
+
+func doVolumeMigration(d *Daemon, poolName string, req *api.StorageVolumesPost) Response {
+	// Validate migration mode
+	if req.Source.Mode != "pull" && req.Source.Mode != "push" {
+		return NotImplemented
+	}
+
+	storage, err := storagePoolVolumeDBCreateInternal(d.State(), poolName, req)
 	if err != nil {
 		return InternalError(err)
 	}
 
-	return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/storage-pools/%s/volumes/%s", version.APIVersion, poolName, apiEndpoint))
+	// create new certificate
+	var cert *x509.Certificate
+	if req.Source.Certificate != "" {
+		certBlock, _ := pem.Decode([]byte(req.Source.Certificate))
+		if certBlock == nil {
+			return InternalError(fmt.Errorf("Invalid certificate"))
+		}
+
+		cert, err = x509.ParseCertificate(certBlock.Bytes)
+		if err != nil {
+			return InternalError(err)
+		}
+	}
+
+	config, err := shared.GetTLSConfig("", "", "", cert)
+	if err != nil {
+		return InternalError(err)
+	}
+
+	push := false
+	if req.Source.Mode == "push" {
+		push = true
+	}
+
+	migrationArgs := MigrationSinkArgs{
+		Url: req.Source.Operation,
+		Dialer: websocket.Dialer{
+			TLSClientConfig: config,
+			NetDial:         shared.RFC3493Dialer},
+		Secrets: req.Source.Websockets,
+		Push:    push,
+		Storage: storage,
+	}
+
+	sink, err := NewStorageMigrationSink(&migrationArgs)
+	if err != nil {
+		return InternalError(err)
+	}
+
+	resources := map[string][]string{}
+	resources["storage_volumes"] = []string{fmt.Sprintf("%s/volumes/custom/%s", poolName, req.Name)}
+
+	run := func(op *operation) error {
+		// And finally run the migration.
+		err = sink.DoStorage(op)
+		if err != nil {
+			logger.Error("Error during migration sink", log.Ctx{"err": err})
+			return fmt.Errorf("Error transferring storage volume: %s", err)
+		}
+
+		return nil
+	}
+
+	var op *operation
+	if push {
+		op, err = operationCreate(d.cluster, operationClassWebsocket, "Creating storage volume", resources, sink.Metadata(), run, nil, sink.Connect)
+		if err != nil {
+			return InternalError(err)
+		}
+	} else {
+		op, err = operationCreate(d.cluster, operationClassTask, "Copying storage volume", resources, nil, run, nil, nil)
+		if err != nil {
+			return InternalError(err)
+		}
+	}
+
+	return OperationResponse(op)
 }
 
 var storagePoolVolumesTypeCmd = Command{name: "storage-pools/{name}/volumes/{type}", get: storagePoolVolumesTypeGet, post: storagePoolVolumesTypePost}
@@ -213,6 +328,10 @@ func storagePoolVolumeTypePost(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("No name provided"))
 	}
 
+	if strings.Contains(req.Name, "/") {
+		return BadRequest(fmt.Errorf("Storage volume names may not contain slashes"))
+	}
+
 	// We currently only allow to create storage volumes of type
 	// storagePoolVolumeTypeCustom. So check, that nothing else was
 	// requested.
@@ -222,29 +341,146 @@ func storagePoolVolumeTypePost(d *Daemon, r *http.Request) Response {
 
 	// Retrieve ID of the storage pool (and check if the storage pool
 	// exists).
-	poolID, err := d.db.StoragePoolGetID(poolName)
+	var poolID int64
+	if req.Pool != "" {
+		poolID, err = d.cluster.StoragePoolGetID(req.Pool)
+	} else {
+		poolID, err = d.cluster.StoragePoolGetID(poolName)
+	}
 	if err != nil {
 		return SmartError(err)
 	}
 
-	// Check that the name isn't already in use.
-	_, err = d.db.StoragePoolVolumeGetTypeID(req.Name,
-		storagePoolVolumeTypeCustom, poolID)
-	if err == nil || err != nil && err != db.NoSuchObjectError {
-		return Conflict
+	// We need to restore the body of the request since it has already been
+	// read, and if we forwarded it now no body would be written out.
+	buf := bytes.Buffer{}
+	err = json.NewEncoder(&buf).Encode(req)
+	if err != nil {
+		return SmartError(err)
+	}
+	r.Body = shared.BytesReadCloser{Buf: &buf}
+
+	response := ForwardedResponseIfTargetIsRemote(d, r)
+	if response != nil {
+		return response
+	}
+
+	// Convert the volume type name to our internal integer representation.
+	volumeType, err := storagePoolVolumeTypeNameToType(volumeTypeName)
+	if err != nil {
+		return BadRequest(err)
+	}
+
+	response = ForwardedResponseIfVolumeIsRemote(d, r, poolID, volumeName, volumeType)
+	if response != nil {
+		return response
 	}
 
 	s, err := storagePoolVolumeInit(d.State(), poolName, volumeName, storagePoolVolumeTypeCustom)
 	if err != nil {
-		return SmartError(err)
+		return InternalError(err)
 	}
 
-	err = s.StoragePoolVolumeRename(req.Name)
+	// This is a migration request so send back requested secrets
+	if req.Migration {
+		ws, err := NewStorageMigrationSource(s)
+		if err != nil {
+			return InternalError(err)
+		}
+
+		resources := map[string][]string{}
+		resources["storage_volumes"] = []string{fmt.Sprintf("%s/volumes/custom/%s", poolName, volumeName)}
+
+		if req.Target != nil {
+			// Push mode
+			err := ws.ConnectStorageTarget(*req.Target)
+			if err != nil {
+				return InternalError(err)
+			}
+
+			op, err := operationCreate(d.cluster, operationClassTask, "Migrating storage volume", resources, nil, ws.DoStorage, nil, nil)
+			if err != nil {
+				return InternalError(err)
+			}
+
+			return OperationResponse(op)
+		}
+
+		// Pull mode
+		op, err := operationCreate(d.cluster, operationClassWebsocket, "Migrating storage volume", resources, ws.Metadata(), ws.DoStorage, nil, ws.Connect)
+		if err != nil {
+			return InternalError(err)
+		}
+
+		return OperationResponse(op)
+	}
+
+	// Check that the name isn't already in use.
+	_, err = d.cluster.StoragePoolNodeVolumeGetTypeID(req.Name,
+		storagePoolVolumeTypeCustom, poolID)
+	if err == nil || err != nil && err != db.ErrNoSuchObject {
+		return Conflict
+	}
+
+	doWork := func() error {
+		ctsUsingVolume, err := storagePoolVolumeUsedByRunningContainersWithProfilesGet(d.State(), poolName, volumeName, storagePoolVolumeTypeNameCustom, true)
+		if err != nil {
+			return err
+		}
+		if len(ctsUsingVolume) > 0 {
+			return fmt.Errorf("Volume is still in use by running containers")
+		}
+
+		err = storagePoolVolumeUpdateUsers(d, poolName, volumeName, req.Pool, req.Name)
+		if err != nil {
+			return err
+		}
+
+		if req.Pool == "" || req.Pool == poolName {
+			err := s.StoragePoolVolumeRename(req.Name)
+			if err != nil {
+				storagePoolVolumeUpdateUsers(d, req.Pool, req.Name, poolName, volumeName)
+				return err
+			}
+		} else {
+			moveReq := api.StorageVolumesPost{}
+			moveReq.Name = req.Name
+			moveReq.Type = "custom"
+			moveReq.Source.Name = volumeName
+			moveReq.Source.Pool = poolName
+			err := storagePoolVolumeCreateInternal(d.State(), req.Pool, &moveReq)
+			if err != nil {
+				storagePoolVolumeUpdateUsers(d, req.Pool, req.Name, poolName, volumeName)
+				return err
+			}
+			err = s.StoragePoolVolumeDelete()
+			if err != nil {
+				return err
+			}
+		}
+
+		return nil
+	}
+
+	if req.Pool == "" {
+		err = doWork()
+		if err != nil {
+			return SmartError(err)
+		}
+
+		return SyncResponseLocation(true, nil, fmt.Sprintf("/%s/storage-pools/%s/volumes/%s", version.APIVersion, poolName, storagePoolVolumeAPIEndpointCustom))
+	}
+
+	run := func(op *operation) error {
+		return doWork()
+	}
+
+	op, err := operationCreate(d.cluster, operationClassTask, "Moving storage volume", nil, nil, run, nil, nil)
 	if err != nil {
 		return InternalError(err)
 	}
 
-	return EmptySyncResponse
+	return OperationResponse(op)
 }
 
 // /1.0/storage-pools/{pool}/volumes/{type}/{name}
@@ -272,13 +508,23 @@ func storagePoolVolumeTypeGet(d *Daemon, r *http.Request) Response {
 
 	// Get the ID of the storage pool the storage volume is supposed to be
 	// attached to.
-	poolID, err := d.db.StoragePoolGetID(poolName)
+	poolID, err := d.cluster.StoragePoolGetID(poolName)
 	if err != nil {
 		return SmartError(err)
 	}
 
+	response := ForwardedResponseIfTargetIsRemote(d, r)
+	if response != nil {
+		return response
+	}
+
+	response = ForwardedResponseIfVolumeIsRemote(d, r, poolID, volumeName, volumeType)
+	if response != nil {
+		return response
+	}
+
 	// Get the storage volume.
-	_, volume, err := d.db.StoragePoolVolumeGetType(volumeName, volumeType, poolID)
+	_, volume, err := d.cluster.StoragePoolNodeVolumeGetType(volumeName, volumeType, poolID)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -316,13 +562,23 @@ func storagePoolVolumeTypePut(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("invalid storage volume type %s", volumeTypeName))
 	}
 
-	poolID, pool, err := d.db.StoragePoolGet(poolName)
+	poolID, pool, err := d.cluster.StoragePoolGet(poolName)
 	if err != nil {
 		return SmartError(err)
 	}
 
+	response := ForwardedResponseIfTargetIsRemote(d, r)
+	if response != nil {
+		return response
+	}
+
+	response = ForwardedResponseIfVolumeIsRemote(d, r, poolID, volumeName, volumeType)
+	if response != nil {
+		return response
+	}
+
 	// Get the existing storage volume.
-	_, volume, err := d.db.StoragePoolVolumeGetType(volumeName, volumeType, poolID)
+	_, volume, err := d.cluster.StoragePoolNodeVolumeGetType(volumeName, volumeType, poolID)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -378,13 +634,23 @@ func storagePoolVolumeTypePatch(d *Daemon, r *http.Request) Response {
 
 	// Get the ID of the storage pool the storage volume is supposed to be
 	// attached to.
-	poolID, pool, err := d.db.StoragePoolGet(poolName)
+	poolID, pool, err := d.cluster.StoragePoolGet(poolName)
 	if err != nil {
 		return SmartError(err)
 	}
 
+	response := ForwardedResponseIfTargetIsRemote(d, r)
+	if response != nil {
+		return response
+	}
+
+	response = ForwardedResponseIfVolumeIsRemote(d, r, poolID, volumeName, volumeType)
+	if response != nil {
+		return response
+	}
+
 	// Get the existing storage volume.
-	_, volume, err := d.db.StoragePoolVolumeGetType(volumeName, volumeType, poolID)
+	_, volume, err := d.cluster.StoragePoolNodeVolumeGetType(volumeName, volumeType, poolID)
 	if err != nil {
 		return SmartError(err)
 	}
@@ -449,6 +715,21 @@ func storagePoolVolumeTypeDelete(d *Daemon, r *http.Request) Response {
 		return BadRequest(fmt.Errorf("invalid storage volume type %s", volumeTypeName))
 	}
 
+	response := ForwardedResponseIfTargetIsRemote(d, r)
+	if response != nil {
+		return response
+	}
+
+	poolID, _, err := d.cluster.StoragePoolGet(poolName)
+	if err != nil {
+		return SmartError(err)
+	}
+
+	response = ForwardedResponseIfVolumeIsRemote(d, r, poolID, volumeName, volumeType)
+	if response != nil {
+		return response
+	}
+
 	switch volumeType {
 	case storagePoolVolumeTypeCustom:
 		// allowed
