@@ -8,6 +8,7 @@ import (
 	"github.com/lxc/lxd/lxd/db"
 	"github.com/lxc/lxd/lxd/state"
 	"github.com/lxc/lxd/shared"
+	"github.com/lxc/lxd/shared/api"
 	"github.com/lxc/lxd/shared/version"
 )
 
@@ -151,14 +152,14 @@ func storagePoolVolumeUpdate(state *state.State, poolName string, volumeName str
 		s.SetStoragePoolVolumeWritable(&newWritable)
 	}
 
-	poolID, err := state.DB.StoragePoolGetID(poolName)
+	poolID, err := state.Cluster.StoragePoolGetID(poolName)
 	if err != nil {
 		return err
 	}
 
 	// Update the database if something changed
 	if len(changedConfig) != 0 || newDescription != oldDescription {
-		err = state.DB.StoragePoolVolumeUpdate(volumeName, volumeType, poolID, newDescription, newConfig)
+		err = state.Cluster.StoragePoolVolumeUpdate(volumeName, volumeType, poolID, newDescription, newConfig)
 		if err != nil {
 			return err
 		}
@@ -172,7 +173,7 @@ func storagePoolVolumeUpdate(state *state.State, poolName string, volumeName str
 
 func storagePoolVolumeUsedByContainersGet(s *state.State, volumeName string,
 	volumeTypeName string) ([]string, error) {
-	cts, err := s.DB.ContainersList(db.CTypeRegular)
+	cts, err := s.Cluster.ContainersList(db.CTypeRegular)
 	if err != nil {
 		return []string{}, err
 	}
@@ -202,6 +203,183 @@ func storagePoolVolumeUsedByContainersGet(s *state.State, volumeName string,
 	return ctsUsingVolume, nil
 }
 
+func storagePoolVolumeUpdateUsers(d *Daemon, oldPoolName string,
+	oldVolumeName string, newPoolName string, newVolumeName string) error {
+
+	s := d.State()
+	// update all containers
+	cts, err := s.Cluster.ContainersList(db.CTypeRegular)
+	if err != nil {
+		return err
+	}
+
+	for _, ct := range cts {
+		c, err := containerLoadByName(s, ct)
+		if err != nil {
+			continue
+		}
+
+		devices := c.LocalDevices()
+		for k := range devices {
+			if devices[k]["type"] != "disk" {
+				continue
+			}
+
+			// Can't be a storage volume.
+			if filepath.IsAbs(devices[k]["source"]) {
+				continue
+			}
+
+			if filepath.Clean(devices[k]["pool"]) != oldPoolName {
+				continue
+			}
+
+			dir, file := filepath.Split(devices[k]["source"])
+			dir = filepath.Clean(dir)
+			if dir != storagePoolVolumeTypeNameCustom {
+				continue
+			}
+
+			file = filepath.Clean(file)
+			if file != oldVolumeName {
+				continue
+			}
+
+			// found entry
+
+			if oldPoolName != newPoolName {
+				devices[k]["pool"] = newPoolName
+			}
+
+			if oldVolumeName != newVolumeName {
+				newSource := newVolumeName
+				if dir != "" {
+					newSource = fmt.Sprintf("%s/%s", storagePoolVolumeTypeNameCustom, newVolumeName)
+				}
+				devices[k]["source"] = newSource
+			}
+		}
+
+		args := db.ContainerArgs{
+			Architecture: c.Architecture(),
+			Description:  c.Description(),
+			Config:       c.LocalConfig(),
+			Devices:      devices,
+			Ephemeral:    c.IsEphemeral(),
+			Profiles:     c.Profiles()}
+
+		err = c.Update(args, false)
+		if err != nil {
+			return err
+		}
+	}
+
+	// update all profiles
+	profiles, err := s.Cluster.Profiles()
+	if err != nil {
+		return err
+	}
+
+	for _, pName := range profiles {
+		id, profile, err := s.Cluster.ProfileGet(pName)
+		if err != nil {
+			return err
+		}
+
+		for k := range profile.Devices {
+			if profile.Devices[k]["type"] != "disk" {
+				continue
+			}
+
+			// Can't be a storage volume.
+			if filepath.IsAbs(profile.Devices[k]["source"]) {
+				continue
+			}
+
+			if filepath.Clean(profile.Devices[k]["pool"]) != oldPoolName {
+				continue
+			}
+
+			dir, file := filepath.Split(profile.Devices[k]["source"])
+			dir = filepath.Clean(dir)
+			if dir != storagePoolVolumeTypeNameCustom {
+				continue
+			}
+
+			file = filepath.Clean(file)
+			if file != oldVolumeName {
+				continue
+			}
+
+			// found entry
+
+			if oldPoolName != newPoolName {
+				profile.Devices[k]["pool"] = newPoolName
+			}
+
+			if oldVolumeName != newVolumeName {
+				newSource := newVolumeName
+				if dir != "" {
+					newSource = fmt.Sprintf("%s/%s", storagePoolVolumeTypeNameCustom, newVolumeName)
+				}
+				profile.Devices[k]["source"] = newSource
+			}
+		}
+
+		pUpdate := api.ProfilePut{}
+		pUpdate.Config = profile.Config
+		pUpdate.Description = profile.Description
+		pUpdate.Devices = profile.Devices
+		err = doProfileUpdate(d, pName, id, profile, pUpdate)
+		if err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+func storagePoolVolumeUsedByRunningContainersWithProfilesGet(s *state.State,
+	poolName string, volumeName string, volumeTypeName string,
+	runningOnly bool) ([]string, error) {
+	cts, err := s.Cluster.ContainersList(db.CTypeRegular)
+	if err != nil {
+		return []string{}, err
+	}
+
+	ctsUsingVolume := []string{}
+	volumeNameWithType := fmt.Sprintf("%s/%s", volumeTypeName, volumeName)
+	for _, ct := range cts {
+		c, err := containerLoadByName(s, ct)
+		if err != nil {
+			continue
+		}
+
+		if runningOnly && !c.IsRunning() {
+			continue
+		}
+
+		for _, dev := range c.ExpandedDevices() {
+			if dev["type"] != "disk" {
+				continue
+			}
+
+			if dev["pool"] != poolName {
+				continue
+			}
+
+			// Make sure that we don't compare against stuff like
+			// "container////bla" but only against "container/bla".
+			cleanSource := filepath.Clean(dev["source"])
+			if cleanSource == volumeName || cleanSource == volumeNameWithType {
+				ctsUsingVolume = append(ctsUsingVolume, ct)
+			}
+		}
+	}
+
+	return ctsUsingVolume, nil
+}
+
 // volumeUsedBy = append(volumeUsedBy, fmt.Sprintf("/%s/containers/%s", version.APIVersion, ct))
 func storagePoolVolumeUsedByGet(s *state.State, volumeName string, volumeTypeName string) ([]string, error) {
 	// Handle container volumes
@@ -233,7 +411,7 @@ func storagePoolVolumeUsedByGet(s *state.State, volumeName string, volumeTypeNam
 			fmt.Sprintf("/%s/containers/%s", version.APIVersion, ct))
 	}
 
-	profiles, err := profilesUsingPoolVolumeGetNames(s.DB, volumeName, volumeTypeName)
+	profiles, err := profilesUsingPoolVolumeGetNames(s.Cluster, volumeName, volumeTypeName)
 	if err != nil {
 		return []string{}, err
 	}
@@ -249,7 +427,7 @@ func storagePoolVolumeUsedByGet(s *state.State, volumeName string, volumeTypeNam
 	return volumeUsedBy, nil
 }
 
-func profilesUsingPoolVolumeGetNames(db *db.Node, volumeName string, volumeType string) ([]string, error) {
+func profilesUsingPoolVolumeGetNames(db *db.Cluster, volumeName string, volumeType string) ([]string, error) {
 	usedBy := []string{}
 
 	profiles, err := db.Profiles()
@@ -302,14 +480,14 @@ func storagePoolVolumeDBCreate(s *state.State, poolName string, volumeName, volu
 	}
 
 	// Load storage pool the volume will be attached to.
-	poolID, poolStruct, err := s.DB.StoragePoolGet(poolName)
+	poolID, poolStruct, err := s.Cluster.StoragePoolGet(poolName)
 	if err != nil {
 		return err
 	}
 
 	// Check that a storage volume of the same storage volume type does not
 	// already exist.
-	volumeID, _ := s.DB.StoragePoolVolumeGetTypeID(volumeName, volumeType, poolID)
+	volumeID, _ := s.Cluster.StoragePoolNodeVolumeGetTypeID(volumeName, volumeType, poolID)
 	if volumeID > 0 {
 		return fmt.Errorf("a storage volume of type %s does already exist", volumeTypeName)
 	}
@@ -331,7 +509,7 @@ func storagePoolVolumeDBCreate(s *state.State, poolName string, volumeName, volu
 	}
 
 	// Create the database entry for the storage volume.
-	_, err = s.DB.StoragePoolVolumeCreate(volumeName, volumeDescription, volumeType, poolID, volumeConfig)
+	_, err = s.Cluster.StoragePoolVolumeCreate(volumeName, volumeDescription, volumeType, poolID, volumeConfig)
 	if err != nil {
 		return fmt.Errorf("Error inserting %s of type %s into database: %s", poolName, volumeTypeName, err)
 	}
@@ -339,29 +517,75 @@ func storagePoolVolumeDBCreate(s *state.State, poolName string, volumeName, volu
 	return nil
 }
 
-func storagePoolVolumeCreateInternal(state *state.State, poolName string, volumeName, volumeDescription string, volumeTypeName string, volumeConfig map[string]string) error {
+func storagePoolVolumeDBCreateInternal(state *state.State, poolName string, vol *api.StorageVolumesPost) (storage, error) {
+	volumeName := vol.Name
+	volumeDescription := vol.Description
+	volumeTypeName := vol.Type
+	volumeConfig := vol.Config
+
+	if vol.Source.Name != "" {
+		// Initialize instance of new pool to translate properties
+		// between storage drivers.
+		s, err := storagePoolInit(state, poolName)
+		if err != nil {
+			return nil, err
+		}
+
+		driver := s.GetStorageTypeName()
+		newConfig, err := storageVolumePropertiesTranslate(vol.Config, driver)
+		if err != nil {
+			return nil, err
+		}
+
+		vol.Config = newConfig
+		volumeConfig = newConfig
+	}
+
+	// Create database entry for new storage volume.
 	err := storagePoolVolumeDBCreate(state, poolName, volumeName, volumeDescription, volumeTypeName, volumeConfig)
 	if err != nil {
-		return err
+		return nil, err
 	}
 
 	// Convert the volume type name to our internal integer representation.
+	poolID, err := state.Cluster.StoragePoolGetID(poolName)
+	if err != nil {
+		return nil, err
+	}
+
 	volumeType, err := storagePoolVolumeTypeNameToType(volumeTypeName)
 	if err != nil {
-		return err
+		state.Cluster.StoragePoolVolumeDelete(volumeName, volumeType, poolID)
+		return nil, err
 	}
 
+	// Initialize new storage volume on the target storage pool.
 	s, err := storagePoolVolumeInit(state, poolName, volumeName, volumeType)
+	if err != nil {
+		state.Cluster.StoragePoolVolumeDelete(volumeName, volumeType, poolID)
+		return nil, err
+	}
+
+	return s, nil
+}
+
+func storagePoolVolumeCreateInternal(state *state.State, poolName string, vol *api.StorageVolumesPost) error {
+	s, err := storagePoolVolumeDBCreateInternal(state, poolName, vol)
 	if err != nil {
 		return err
 	}
 
-	poolID, _, _ := s.GetContainerPoolInfo()
-
-	// Create storage volume.
-	err = s.StoragePoolVolumeCreate()
+	if vol.Source.Name == "" {
+		err = s.StoragePoolVolumeCreate()
+	} else {
+		err = s.StoragePoolVolumeCopy(&vol.Source)
+	}
 	if err != nil {
-		state.DB.StoragePoolVolumeDelete(volumeName, volumeType, poolID)
+		poolID, _, _ := s.GetContainerPoolInfo()
+		volumeType, err := storagePoolVolumeTypeNameToType(vol.Type)
+		if err == nil {
+			state.Cluster.StoragePoolVolumeDelete(vol.Name, volumeType, poolID)
+		}
 		return err
 	}
 
