@@ -1,9 +1,9 @@
-// +build linux darwin
+// +build linux darwin freebsd
 
 package logging
 
 import (
-	log "gopkg.in/inconshreveable/log15.v2"
+	log "github.com/lxc/lxd/shared/log15"
 )
 
 // getSystemHandler on Linux writes messages to syslog.
