@@ -4,13 +4,15 @@ import (
 	"bufio"
 	"bytes"
 	"crypto/rand"
+	"crypto/sha256"
+	"crypto/sha512"
 	"encoding/gob"
 	"encoding/hex"
 	"encoding/json"
 	"fmt"
+	"hash"
 	"io"
 	"io/ioutil"
-	"math"
 	"net/http"
 	"net/url"
 	"os"
@@ -24,6 +26,9 @@ import (
 	"strings"
 	"time"
 	"unicode"
+
+	"github.com/lxc/lxd/shared/cancel"
+	"github.com/lxc/lxd/shared/ioprogress"
 )
 
 const SnapshotDelimiter = "/"
@@ -116,7 +121,8 @@ func HostPath(path string) string {
 
 	// Check if we're running in a snap package
 	snap := os.Getenv("SNAP")
-	if snap == "" {
+	snapName := os.Getenv("SNAP_NAME")
+	if snap == "" || snapName != "lxd" {
 		return path
 	}
 
@@ -138,7 +144,7 @@ func HostPath(path string) string {
 
 	// Check if the path is already snap-aware
 	for _, prefix := range []string{"/dev", "/snap", "/var/snap", "/var/lib/snapd"} {
-		if strings.HasPrefix(path, prefix) {
+		if path == prefix || strings.HasPrefix(path, fmt.Sprintf("%s/", prefix)) {
 			return path
 		}
 	}
@@ -304,11 +310,9 @@ func ReadStdin() ([]byte, error) {
 	return line, nil
 }
 
-func WriteAll(w io.Writer, buf []byte) error {
-	return WriteAllBuf(w, bytes.NewBuffer(buf))
-}
+func WriteAll(w io.Writer, data []byte) error {
+	buf := bytes.NewBuffer(data)
 
-func WriteAllBuf(w io.Writer, buf *bytes.Buffer) error {
 	toWrite := int64(buf.Len())
 	for {
 		n, err := io.Copy(w, buf)
@@ -402,19 +406,6 @@ func ExtractSnapshotName(name string) string {
 	return strings.SplitN(name, SnapshotDelimiter, 2)[1]
 }
 
-func ReadDir(p string) ([]string, error) {
-	ents, err := ioutil.ReadDir(p)
-	if err != nil {
-		return []string{}, err
-	}
-
-	var ret []string
-	for _, ent := range ents {
-		ret = append(ret, ent.Name())
-	}
-	return ret, nil
-}
-
 func MkdirAllOwner(path string, perm os.FileMode, uid int, gid int) error {
 	// This function is a slightly modified version of MkdirAll from the Go standard library.
 	// https://golang.org/src/os/path.go?s=488:535#L9
@@ -502,6 +493,20 @@ func IsTrue(value string) bool {
 	return false
 }
 
+func IsUnixDev(path string) bool {
+	stat, err := os.Stat(path)
+	if err != nil {
+		return false
+
+	}
+
+	if (stat.Mode() & os.ModeDevice) == 0 {
+		return false
+	}
+
+	return true
+}
+
 func IsBlockdev(fm os.FileMode) bool {
 	return ((fm&os.ModeDevice != 0) && (fm&os.ModeCharDevice == 0))
 }
@@ -871,14 +876,6 @@ func TimeIsSet(ts time.Time) bool {
 	return true
 }
 
-func Round(x float64) int64 {
-	if x < 0 {
-		return int64(math.Ceil(x - 0.5))
-	}
-
-	return int64(math.Floor(x + 0.5))
-}
-
 // WriteTempFile creates a temp file with the specified content
 func WriteTempFile(dir string, prefix string, content string) (string, error) {
 	f, err := ioutil.TempFile(dir, prefix)
@@ -902,3 +899,68 @@ func EscapePathFstab(path string) string {
 		"\\", "\\\\")
 	return r.Replace(path)
 }
+
+func DownloadFileSha256(httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.Canceler, filename string, url string, hash string, target io.WriteSeeker) (int64, error) {
+	return downloadFileSha(httpClient, useragent, progress, canceler, filename, url, hash, sha256.New(), target)
+}
+
+func DownloadFileSha512(httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.Canceler, filename string, url string, hash string, target io.WriteSeeker) (int64, error) {
+	return downloadFileSha(httpClient, useragent, progress, canceler, filename, url, hash, sha512.New(), target)
+}
+
+func downloadFileSha(httpClient *http.Client, useragent string, progress func(progress ioprogress.ProgressData), canceler *cancel.Canceler, filename string, url string, hash string, sha hash.Hash, target io.WriteSeeker) (int64, error) {
+	// Always seek to the beginning
+	target.Seek(0, 0)
+
+	// Prepare the download request
+	req, err := http.NewRequest("GET", url, nil)
+	if err != nil {
+		return -1, err
+	}
+
+	if useragent != "" {
+		req.Header.Set("User-Agent", useragent)
+	}
+
+	// Perform the request
+	r, doneCh, err := cancel.CancelableDownload(canceler, httpClient, req)
+	if err != nil {
+		return -1, err
+	}
+	defer r.Body.Close()
+	defer close(doneCh)
+
+	if r.StatusCode != http.StatusOK {
+		return -1, fmt.Errorf("Unable to fetch %s: %s", url, r.Status)
+	}
+
+	// Handle the data
+	body := r.Body
+	if progress != nil {
+		body = &ioprogress.ProgressReader{
+			ReadCloser: r.Body,
+			Tracker: &ioprogress.ProgressTracker{
+				Length: r.ContentLength,
+				Handler: func(percent int64, speed int64) {
+					if filename != "" {
+						progress(ioprogress.ProgressData{Text: fmt.Sprintf("%s: %d%% (%s/s)", filename, percent, GetByteSizeString(speed, 2))})
+					} else {
+						progress(ioprogress.ProgressData{Text: fmt.Sprintf("%d%% (%s/s)", percent, GetByteSizeString(speed, 2))})
+					}
+				},
+			},
+		}
+	}
+
+	size, err := io.Copy(io.MultiWriter(target, sha), body)
+	if err != nil {
+		return -1, err
+	}
+
+	result := fmt.Sprintf("%x", sha.Sum(nil))
+	if result != hash {
+		return -1, fmt.Errorf("Hash mismatch for %s: %s != %s", url, result, hash)
+	}
+
+	return size, nil
+}
