@@ -9,7 +9,7 @@ ensure_has_localhost_remote() {
 }
 
 ensure_import_testimage() {
-    if ! lxc image alias list | grep -q "^| testimage\s*|.*$"; then
+    if ! lxc image alias list | grep -q "^| testimage\\s*|.*$"; then
         if [ -e "${LXD_TEST_IMAGE:-}" ]; then
             lxc image import "${LXD_TEST_IMAGE}" --alias testimage
         else
