@@ -51,6 +51,10 @@ test_idmap() {
   [ "$(lxc exec idmap -- cat /proc/self/uid_map | awk '{print $3}')" = "${UIDs}" ]
   [ "$(lxc exec idmap -- cat /proc/self/gid_map | awk '{print $3}')" = "${GIDs}" ]
 
+  # Confirm that we don't allow double mappings
+  ! echo "uid $((UID_BASE+1)) 1000" | lxc config set idmap raw.idmap - || false
+  ! echo "gid $((GID_BASE+1)) 1000" | lxc config set idmap raw.idmap - || false
+
   # Convert container to isolated and confirm it's not using the first range
   lxc config set idmap security.idmap.isolated true
   lxc restart idmap --force
@@ -67,7 +71,17 @@ test_idmap() {
   [ "$(lxc exec idmap -- cat /proc/self/uid_map | awk '{print $3}')" = "100000" ]
   [ "$(lxc exec idmap -- cat /proc/self/gid_map | awk '{print $3}')" = "100000" ]
 
+  # Test using a custom base
+  lxc config set idmap security.idmap.base $((UID_BASE+12345))
+  lxc config set idmap security.idmap.size 110000
+  lxc restart idmap --force
+  [ "$(lxc exec idmap -- cat /proc/self/uid_map | awk '{print $2}')" = "$((UID_BASE+12345))" ]
+  [ "$(lxc exec idmap -- cat /proc/self/gid_map | awk '{print $2}')" = "$((GID_BASE+12345))" ]
+  [ "$(lxc exec idmap -- cat /proc/self/uid_map | awk '{print $3}')" = "110000" ]
+  [ "$(lxc exec idmap -- cat /proc/self/gid_map | awk '{print $3}')" = "110000" ]
+
   # Switch back to full LXD range
+  lxc config unset idmap security.idmap.base
   lxc config unset idmap security.idmap.isolated
   lxc config unset idmap security.idmap.size
   lxc restart idmap --force
