@@ -27,11 +27,17 @@ test_image_expiry() {
   lxc_remote image list l2: | grep -q "${fpbrief}"
 
   # Override the upload date
-  sqlite3 "${LXD2_DIR}/lxd.db" "UPDATE images SET last_use_date='$(date --rfc-3339=seconds -u -d "2 days ago")' WHERE fingerprint='${fp}'"
+  LXD_DIR="$LXD2_DIR" lxd sql "UPDATE images SET last_use_date='$(date --rfc-3339=seconds -u -d "2 days ago")' WHERE fingerprint='${fp}'" | grep -q "Rows affected: 1"
 
   # Trigger the expiry
   lxc_remote config set l2: images.remote_cache_expiry 1
-  sleep 5
+
+  # shellcheck disable=SC2034
+  for i in $(seq 20); do
+    sleep 1
+    ! lxc_remote image list l2: | grep -q "${fpbrief}" && break
+  done
+
   ! lxc_remote image list l2: | grep -q "${fpbrief}" || false
 
   # Cleanup and reset
